@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2010, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -136,6 +136,8 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
                      valid
 */
 		       ){
+#pragma omp parallel
+  {
   FIDX el, d, i, j, k, r, s;
   int  err;
 
@@ -159,6 +161,8 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
   double *dotfxc;        /* time derivative of rhs function f(.) at
 			    the integration point */
   double dotfxc_mem;     /* memory in case dotfxc is used */
+  double *rhsloc;        /* element rhs */
+  double *rhsdotloc;     /* element rhsdot */
   double *elK;           /* element stiffness matrix */
   double *elM;           /* element mass matrix */
   double *elAB;          /* temporary storage for cA_M*elM and the
@@ -240,7 +244,7 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
       break;
     default:
       fprintf(stderr,"assem_conv_diff_tx: unknown type=%d\n",type);
-      return FAIL;
+      FEINS_FAILURE_ACTION
     }
 
   m_type=(FIDX) type;
@@ -261,7 +265,7 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
       /* cry */
       fprintf(stderr,
 	      "assem_conv_diff_tx: dim >2 not implemented (Jacinv+?)\n");
-      return FAIL;
+      FEINS_FAILURE_ACTION
     }
 
   if ( ((K!=NULL)&&((*K).row_nr!=vx_nr))
@@ -273,7 +277,7 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
       /* cry */
       fprintf(stderr,
 	      "assem_conv_diff_tx: size of K,M,A,B wrong???\n");
-      return FAIL; 
+      FEINS_FAILURE_ACTION
     }
     
   
@@ -286,7 +290,7 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
 	  {
 	    fprintf(stderr,
 		    "assem_conv_diff_tx: eps not symmetric???\n");
-	    return FAIL; 
+	    FEINS_FAILURE_ACTION
 	  }
       }
   {
@@ -310,7 +314,7 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
       {
 	fprintf(stderr,
 		"assem_conv_diff_tx: error determining eigenvalues of eps\n");
-	return FAIL; 
+	FEINS_FAILURE_ACTION
       }
     free(evs);
     free(work);
@@ -320,7 +324,7 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
     {
       fprintf(stderr,
 	      "assem_conv_diff_tx: eps min eigenvalue <=0\n");
-      return FAIL; 
+      FEINS_FAILURE_ACTION
     }
 
 
@@ -369,7 +373,12 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
 
   TRY_MALLOC( gradc_glob, dim*bas_nc, double, assem_conv_diff_tx);
 
+  TRY_MALLOC( rhsloc, bas_n, double, assem_conv_diff_tx);
+  TRY_MALLOC( rhsdotloc, bas_n, double, assem_conv_diff_tx);
+
   /* clear K,M,A,B */
+#pragma omp master
+  {
   if (K!=NULL)
     {
       sparse_empty(K);
@@ -392,23 +401,32 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
     {
       rhs->V[i]=0.0;
     }
+  *maxPe=0.0;
+  } /* END OPM master */
 
   /* clear rhsdot */
   if (rhsdot!=NULL)
     {
+#pragma omp master
+      {
       for (i=0; i<vx_nr; i++)
 	{
 	  rhsdot->V[i]=0.0;
 	}
+      }  /* END OPM master */
       dotfxc=&dotfxc_mem;
     }
   else
     {
       dotfxc=NULL;
     }
-  *maxPe=0.0;
+
+  /* wait till all threads have finished init phase */
+#pragma omp barrier
+
 
   /* loop over all elements */
+#pragma omp for schedule(static)
   for (el=0; el<el_nr; el++)
     {
       /* determine the mesh peclet number for this element */
@@ -494,8 +512,13 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
 
       /* this elements Peclet number Pe */
       pe=0.5*h*abs_bc/min_eps;
+
       /* update the maximum Pe */
-      if (*maxPe<pe) *maxPe=pe;
+#pragma omp critical
+      {
+	if (*maxPe<pe) *maxPe=pe;
+      } /* end OMP critical */
+
       /* stabilization parameter beta */
       if (abs_bc!=0.0)
 	{
@@ -536,6 +559,13 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
 	      elM[i]=0.0;
 	    }
 	}
+      /* set rhsloc and rhsdotloc to zero */
+      for (i=0; i<bas_n; i++)
+	{
+	  rhsloc[i]=0.0;
+	  rhsdotloc[i]=0.0;
+	}
+
 
       /* loop over all integration points */
       for (k=0; k<iform.num_points; k++)
@@ -692,14 +722,14 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
 		    }
 		}
 	      /* compute rhs */
-	      (*rhs).V[(*m).elem[el*el_w+i]] +=
+	      rhsloc[i] +=
 		fxc *( phi[k*bas_n+i] + beta*b_gradphi_i)
 		* weight ;
 
 	      /* compute rhsdot */
 	      if (rhsdot!=NULL)
 		{
-		  (*rhsdot).V[(*m).elem[el*el_w+i]] +=
+		  rhsdotloc[i] +=
 		    (*dotfxc) *( phi[k*bas_n+i] + beta*b_gradphi_i)
 		    * weight ;
 		}
@@ -707,29 +737,57 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
 	} /* end loop intergration points */
       
       /***************************************************************
-       * elK is ready, now assemble K                                *
+       * elK is ready, now assemble rhs, K, M, A, B                  *
        ***************************************************************/
 
+
+#pragma omp critical
+      {
+	for (i=0; i<bas_n; i++)
+	  {
+	    (*rhs).V[(*m).elem[el*el_w+i]] += rhsloc[i];
+	  }
+      } /* end OMP critical */
+
+#pragma omp critical
+      {
+	if (rhsdot!=NULL)
+	  {
+	    for (i=0; i<bas_n; i++)
+	      {
+		(*rhsdot).V[(*m).elem[el*el_w+i]] += rhsdotloc[i];
+	      }
+	  }
+      } /* end OMP critical */
+
       if (K!=NULL)
 	{
-	  err=sparse_add_local( K, NoTrans,
-				bas_n, &(*m).elem[el*el_w+l_mctxELNOD1], 
-				bas_n, &(*m).elem[el*el_w+l_mctxELNOD1],
-				elK, bas_n );
-	  FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
+#pragma omp critical
+	  {
+	    err=sparse_add_local( K, NoTrans,
+				  bas_n, &(*m).elem[el*el_w+l_mctxELNOD1], 
+				  bas_n, &(*m).elem[el*el_w+l_mctxELNOD1],
+				  elK, bas_n );
+	    FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
+	  } /* end OMP critical */
 	}
 
       if (M!=NULL)
 	{
-	  err=sparse_add_local( M, NoTrans,
-				bas_n, &(*m).elem[el*el_w+l_mctxELNOD1], 
-				bas_n, &(*m).elem[el*el_w+l_mctxELNOD1],
-				elM, bas_n );
-	  FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
+#pragma omp critical
+	  {
+	    err=sparse_add_local( M, NoTrans,
+				  bas_n, &(*m).elem[el*el_w+l_mctxELNOD1], 
+				  bas_n, &(*m).elem[el*el_w+l_mctxELNOD1],
+				  elM, bas_n );
+	    FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
+	  } /* end OMP critical */
 	}
 
       if (A!=NULL)
 	{
+#pragma omp critical
+	  {
 	  if (cA_M!=0.0)
 	    {
 	      for (i=0; i<bas_n_2; i++)
@@ -754,9 +812,12 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
 				    elAB, bas_n );
 	      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
 	    }
+	  } /* end OMP critical */
 	}
       if (B!=NULL)
 	{
+#pragma omp critical
+	  {
 	  if (cB_M!=0.0)
 	    {
 	      for (i=0; i<bas_n_2; i++)
@@ -781,6 +842,7 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
 				    elAB, bas_n );
 	      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_conv_diff_tx);
 	    }
+	  } /* end OMP critical */
 	}
 
     } /* end loop elements */
@@ -805,6 +867,7 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
 
   free_intdata (&iform);
 
+  } /* END OMP parallel */
 
   return SUCCESS;
 }
