@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -62,14 +62,14 @@ TO_HEADER:
 #include "assembly.h"
 
 /*FUNCTION*/
-int assem_poison_t1(struct sparse *K, struct vector *rhs,
+int assem_poisson_t1(struct sparse *K, struct vector *rhs,
 		    struct vector *u0, struct projector1 *P, 
 		    struct mesh *m
 /* performs the assembly of the stiffness matrix K and the right hand
    side vector rhs which result from finite element discretisation of
-   the Poison equation on the mesh m, such that 
+   the Poisson equation on the mesh m, such that 
        K u = rhs
-   defines the approximate solution u of the Poison equation
+   defines the approximate solution u of the Poisson equation
        Laplace(u) = f
    with boundary conditions as given in the mesh m    
    (only p1 triangles for now)
@@ -78,12 +78,13 @@ int assem_poison_t1(struct sparse *K, struct vector *rhs,
 
    Output: K       - stiffness matrix
            rhs     - righthand side vector
-	   u0      - initial guess for the solution of the equation
- 	             system which satisfies the Dirichlet BC
 	   P       - data for a projector which projects onto the
            	     linear subspace which doesn't change the values
            	     of DOFs related to Dirichlet BC
 
+   In/Out: u0      - initial guess for the solution of the equation
+ 	             system, modified to satisfy the Dirichlet BC
+
    Return: SUCCESS - success
            FAIL    - failure, see error message, output will not be
                      valid
@@ -119,7 +120,7 @@ int assem_poison_t1(struct sparse *K, struct vector *rhs,
   if (err!=SUCCESS)
     {
       fprintf( stderr,
-	       "cubature_bases returned error in assem_poison_t1!\n");
+	       "cubature_bases returned error in assem_poisson_t1!\n");
       return FAIL;
     }
 
@@ -140,20 +141,19 @@ int assem_poison_t1(struct sparse *K, struct vector *rhs,
   fu_w  = (*m).fu_w;
 
   /* allocate memory for elK */
-  TRY_MALLOC( elK, bas_n*bas_n, double, assem_poison_t1);
+  TRY_MALLOC( elK, bas_n*bas_n, double, assem_poisson_t1);
   /* allocate memory for Jac, Jacinf */
-  TRY_MALLOC( Jac, dim*dim, double, assem_poison_t1);
-  TRY_MALLOC( Jacinv, dim*dim, double, assem_poison_t1);
+  TRY_MALLOC( Jac, dim*dim, double, assem_poisson_t1);
+  TRY_MALLOC( Jacinv, dim*dim, double, assem_poisson_t1);
   /* allocate memory for Jinvgrad */
-  TRY_MALLOC( Jinvgrad, dim*bas_n, double, assem_poison_t1);
+  TRY_MALLOC( Jinvgrad, dim*bas_n, double, assem_poisson_t1);
 
   /* clear K */
   sparse_empty(K);
-  /* clear rhs, u0, P */
+  /* clear rhs, P */
   for (i=0; i<vx_nr; i++)
     {
       (*rhs).V[i]=0.0;
-      (*u0).V[i] =0.0;
       (*P).V[i]=0;
     }
 
@@ -230,7 +230,7 @@ int assem_poison_t1(struct sparse *K, struct vector *rhs,
 	      err=mesh_func_eval ( m, (*m).face[el*fc_w+MCT1FCRHSF],
 				   x_k, 0.0, 1, &fx_k, NULL, NULL);
 	      FUNCTION_FAILURE_HANDLE(err,mesh_func_eval,
-				      assem_poison_t1);
+				      assem_poisson_t1);
 	      for (i=0; i<bas_n; i++)
 		(*rhs).V[(*m).elem[el*el_w+i]]+=
 		  iform.weights[k]*fabs(detJac)*fx_k*phi[k*bas_n+i];
@@ -246,13 +246,13 @@ int assem_poison_t1(struct sparse *K, struct vector *rhs,
 			   bas_n, &((*m).elem[el*el_w+MCT1ELNOD1]),
 			   bas_n, &((*m).elem[el*el_w+MCT1ELNOD1]),
 			   elK, bas_n );
-      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_poison_t1);
+      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_poisson_t1);
 	       
     }
   /* end loop over all elements */
 
 #ifdef FEINS_have_warning 
-#warning "poison_T1: Neumann BC not accounted for in rhs yet!"
+#warning "poisson_T1: Neumann BC not accounted for in rhs yet!"
 #endif
   /* first Neumann BC, otherwise they would collide with Dirichlet BC
   */
@@ -282,7 +282,7 @@ int assem_poison_t1(struct sparse *K, struct vector *rhs,
 	  if ((err1!=SUCCESS)||(err2)!=SUCCESS)
 	    {
 	      fprintf(stderr,
-		      "assem_poison_t1: function_eval error in DiriBC\n");
+		      "assem_poisson_t1: function_eval error in DiriBC\n");
 	      return FAIL;
 	    }
 
@@ -313,7 +313,7 @@ int assem_poison_t1(struct sparse *K, struct vector *rhs,
     {
       err = sparse_convert_compressed_row( K );
       FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row, 
-			       assem_poison_t1 );
+			       assem_poisson_t1 );
     }
 
   /* free local data */
@@ -329,14 +329,14 @@ int assem_poison_t1(struct sparse *K, struct vector *rhs,
 
 
 /*FUNCTION*/
-int assem_poison_t2(struct sparse *K, struct vector *rhs,
+int assem_poisson_t2(struct sparse *K, struct vector *rhs,
 		    struct vector *u0, struct projector1 *P,
 		    struct mesh *m
 /* performs the assembly of the stiffness matrix K and the right hand
    side vector rhs which result from finite element discretisation of
-   the Poison equation on the mesh m, such that 
+   the Poisson equation on the mesh m, such that 
        K u = rhs
-   defines the approximate solution u of the Poison equation
+   defines the approximate solution u of the Poisson equation
        Laplace(u) = f
    with boundary conditions as given in the mesh m    
    (p2 triangles)
@@ -345,12 +345,13 @@ int assem_poison_t2(struct sparse *K, struct vector *rhs,
 
    Output: K       - stiffness matrix
            rhs     - righthand side vector
-	   u0      - initial guess for the solution of the equation
- 	             system which satisfies the Dirichlet BC
 	   P       - data for a projector which projects onto the
            	     linear subspace which doesn't change the values
            	     of DOFs related to Dirichlet BC
 
+   In/Out: u0      - initial guess for the solution of the equation
+ 	             system, modified to satisfy the Dirichlet BC
+
    Return: SUCCESS - success
            FAIL    - failure, see error message, output will not be
                      valid
@@ -386,7 +387,7 @@ int assem_poison_t2(struct sparse *K, struct vector *rhs,
   if (err!=SUCCESS)
     {
       fprintf( stderr,
-	       "cubature_bases returned error in assem_poison_t1!\n");
+	       "cubature_bases returned error in assem_poisson_t1!\n");
       return FAIL;
     }
 
@@ -407,20 +408,19 @@ int assem_poison_t2(struct sparse *K, struct vector *rhs,
   fu_w  = (*m).fu_w;
 
   /* allocate memory for elK */
-  TRY_MALLOC( elK, bas_n*bas_n, double, assem_poison_t1);
+  TRY_MALLOC( elK, bas_n*bas_n, double, assem_poisson_t1);
   /* allocate memory for Jac, Jacinf */
-  TRY_MALLOC( Jac, dim*dim, double, assem_poison_t1);
-  TRY_MALLOC( Jacinv, dim*dim, double, assem_poison_t1);
+  TRY_MALLOC( Jac, dim*dim, double, assem_poisson_t1);
+  TRY_MALLOC( Jacinv, dim*dim, double, assem_poisson_t1);
   /* allocate memory for Jinvgrad */
-  TRY_MALLOC( Jinvgrad, dim*bas_n, double, assem_poison_t1);
+  TRY_MALLOC( Jinvgrad, dim*bas_n, double, assem_poisson_t1);
 
   /* clear K */
   sparse_empty(K);
-  /* clear rhs, u0, P */
+  /* clear rhs, P */
   for (i=0; i<vx_nr; i++)
     {
       (*rhs).V[i]=0.0;
-      (*u0).V[i] =0.0;
       (*P).V[i]=0;
     }
 
@@ -500,7 +500,7 @@ int assem_poison_t2(struct sparse *K, struct vector *rhs,
 	      err=mesh_func_eval ( m, (*m).face[el*fc_w+MCT2FCRHSF],
 				   x_k, 0.0, 1, &fx_k, NULL, NULL);
 	      FUNCTION_FAILURE_HANDLE(err,mesh_func_eval,
-				      assem_poison_t2);
+				      assem_poisson_t2);
 	      for (i=0; i<bas_n; i++)
 		(*rhs).V[(*m).elem[el*el_w+i]]+=
 		  iform.weights[k]*fabs(detJac)*fx_k*phi[k*bas_n+i];
@@ -518,12 +518,12 @@ int assem_poison_t2(struct sparse *K, struct vector *rhs,
 			   bas_n, &((*m).elem[el*el_w+MCT2ELNOD1]),
 			   bas_n, &((*m).elem[el*el_w+MCT2ELNOD1]),
 			   elK, bas_n );
-      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_poison_t2);
+      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_poisson_t2);
 		       
     }
   /* end loop over all elements */
 #ifdef FEINS_have_warning 
-#warning "poison_T2: Neumann BC not accounted for in rhs yet!"
+#warning "poisson_T2: Neumann BC not accounted for in rhs yet!"
 #endif
   /* first Neumann BC, otherwise they would collide with Dirichlet BC
   */
@@ -557,7 +557,7 @@ int assem_poison_t2(struct sparse *K, struct vector *rhs,
 	  if (((err1!=SUCCESS)||(err2)!=SUCCESS)||((errm)!=SUCCESS))
 	    {
 	      fprintf(stderr,
-		      "assem_poison_t2: function_eval error in DiriBC\n");
+		      "assem_poisson_t2: function_eval error in DiriBC\n");
 	      return FAIL;
 	    }
 
@@ -589,7 +589,7 @@ int assem_poison_t2(struct sparse *K, struct vector *rhs,
     {
       err = sparse_convert_compressed_row( K );
       FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row, 
-			       assem_poison_t2 );
+			       assem_poisson_t2 );
     }
 
   /* free local data */
@@ -605,10 +605,10 @@ int assem_poison_t2(struct sparse *K, struct vector *rhs,
   
   
 /*FUNCTION*/
-int assem_poison_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
+int assem_poisson_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
 		       struct mesh *m
 /* performs the assembly of the stiffness matrix K which results from
-   finite element discretisation of the Poison equation with linear
+   finite element discretisation of the Poisson equation with linear
    triangle elements (T1) on the (T2) mesh m
 
    Input:  m       - the mesh
@@ -657,7 +657,7 @@ int assem_poison_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
   if (err!=SUCCESS)
     {
       fprintf( stderr,
-	       "assem_poison_t1_t2: cubature_bases returned error!\n");
+	       "assem_poisson_t1_t2: cubature_bases returned error!\n");
       return FAIL;
     }
 
@@ -674,14 +674,14 @@ int assem_poison_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
   el_w  = (*m).el_w;
 
   /* allocate memory for elK */
-  TRY_MALLOC( elK, bas_n*bas_n, double, assem_poison_t1_t2);
+  TRY_MALLOC( elK, bas_n*bas_n, double, assem_poisson_t1_t2);
   /* allocate memory for elDOFS */
-  TRY_MALLOC( elDOFS, bas_n, FIDX, assem_poison_t1_t2);
+  TRY_MALLOC( elDOFS, bas_n, FIDX, assem_poisson_t1_t2);
   /* allocate memory for Jac, Jacinf */
-  TRY_MALLOC( Jac, dim*dim, double, assem_poison_t1_t2);
-  TRY_MALLOC( Jacinv, dim*dim, double, assem_poison_t1_t2);
+  TRY_MALLOC( Jac, dim*dim, double, assem_poisson_t1_t2);
+  TRY_MALLOC( Jacinv, dim*dim, double, assem_poisson_t1_t2);
   /* allocate memory for Jinvgrad */
-  TRY_MALLOC( Jinvgrad, dim*bas_n, double, assem_poison_t1_t2);
+  TRY_MALLOC( Jinvgrad, dim*bas_n, double, assem_poisson_t1_t2);
 
   /* clear K */
   sparse_empty(K);
@@ -694,7 +694,7 @@ int assem_poison_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
 	{
 	  if ((*m).elem[el*el_w+MCT2ELNOD1+i]>=vx_nr1)
 	    {
-	      fprintf(stderr,"assem_poison_t1_t2: error, element contains\n"
+	      fprintf(stderr,"assem_poisson_t1_t2: error, element contains\n"
 		      "vertex %d > vx_nr1=%d\n",
 		      (int) (*m).elem[el*el_w+MCT2ELNOD1+i], (int) vx_nr1);
 	      return FAIL;
@@ -758,7 +758,7 @@ int assem_poison_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
 	  if (elDOFS[i]<0)
 	    {
 	      fprintf(stderr,
-		      "assem_poison_t1_t2: dofs don't match mesh!\n");
+		      "assem_poisson_t1_t2: dofs don't match mesh!\n");
 	      printf("elDOFS[%d]=%d\n", (int) i, (int) elDOFS[i]);
 	      return FAIL;
 	    }
@@ -767,7 +767,7 @@ int assem_poison_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
       /* elK is ready, add to K */
       err=sparse_add_local(K, NoTrans, bas_n, elDOFS, bas_n, elDOFS,
 			   elK, bas_n );
-      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_poison_t1_t2);
+      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_poisson_t1_t2);
 	       
     }
 
@@ -775,7 +775,7 @@ int assem_poison_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
     {
       err = sparse_convert_compressed_row( K );
       FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row, 
-			       assem_poison_t1_t2 );
+			       assem_poisson_t1_t2 );
     }
 
   /* end loop over all elements */
@@ -791,10 +791,364 @@ int assem_poison_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
 }
 
 
+#include "assembly_inline_funs.h"
 
 
+/*FUNCTION*/
+int assem_poisson_e1(struct sparse *K, struct vector *rhs,
+		    struct vector *u0, struct projector1 *P, 
+		    struct mesh *m
+/* performs the assembly of the stiffness matrix K and the right hand
+   side vector rhs which result from finite element discretisation of
+   the Poisson equation on the mesh m, such that 
+       K u = rhs
+   defines the approximate solution u of the Poisson equation
+       Laplace(u) = f
+   with boundary conditions as given in the mesh m    
+   (only p1 tetrahedra for now)
+
+   Input:  m       - the mesh
+
+   Output: K       - stiffness matrix
+           rhs     - righthand side vector
+	   P       - data for a projector which projects onto the
+           	     linear subspace which doesn't change the values
+           	     of DOFs related to Dirichlet BC
+
+   In/Out: u0      - initial guess for the solution of the equation
+ 	             system, modified to satisfy the Dirichlet BC
+
+   Return: SUCCESS - success
+           FAIL    - failure, see error message, output will not be
+                     valid
+*/
+		    ){
+  FIDX el,bd,i,j,k,r;
+  int err;
+
+  FIDX dim=3, bas_n, vx_nr, el_w, vx_w, eg_w, fc_w, vo_w, bd_w, fu_w;
+  FIDX subtypes[1];
+
+  /* for FORTRAN calls: */
+  char fTrans='T', fNoTrans='N';
+  double dhelp, done=1.0, dzero=0.0;
+  int fdim, fbas_n, info;
+
+  struct int_data iform; /* integration formula    */
+  double *elK;           /* element stiffnes matrx */
+  double *Jac;           /* Jacobian of the element mapping later
+			    overwritten by its inverse */
+  double *WORK;          /* workspace for dgetri */
+  int f_LWORK;           /* size of WORK */
+  int *IPIV;             /* vector storing permutation used in
+			    factorizing and inverting the Jacobian */
+			    
+
+  double x_k[3],fx_k;    /* world coordinates of the quadrature point
+			    and a function value at this point */
+  double *Jinvgrad;      /* inverse Jacobian times gradphi */
+  double detJac;         /* determinant of the Jacobian */
+  double *phi, *gradp;
+
+  //double meshvol=0.0;
+
+  /* init */
+  /* get integration formula */
+  subtypes[0]=1;
+
+  err=cubature_bases( dim, 1, tetra, 1, subtypes, &iform); 
+
+  if (err!=SUCCESS)
+    {
+      fprintf( stderr,
+	       "cubature_bases returned error in assem_poisson_e1!\n");
+      return FAIL;
+    }
+
+  /* make phi and gradphi better accessible */
+  phi   = (iform.bases[0]->phi);
+  gradp = (iform.bases[0]->gradphi);
+  bas_n = (iform.bases[0]->num_basis);
+
+  fdim  = (int) dim;
+  fbas_n= (int) bas_n;
+
+  vx_nr = (*m).vx_nr;
+  vx_w  = (*m).vx_w;
+  el_w  = (*m).el_w;
+  eg_w  = (*m).eg_w;
+  fc_w  = (*m).fc_w;
+  vo_w  = (*m).vo_w;
+  bd_w  = (*m).bd_w;
+  fu_w  = (*m).fu_w;
+
+  /* allocate memory for elK */
+  TRY_MALLOC( elK, bas_n*bas_n, double, assem_poisson_e1);
+  /* allocate memory for Jac, Jacinf */
+  TRY_MALLOC( Jac, dim*dim, double, assem_poisson_e1);
+  TRY_MALLOC( IPIV, dim, int, assem_poisson_e1);
+  f_LWORK=5*dim;
+  TRY_MALLOC( WORK, f_LWORK, double, assem_poisson_e1);
+
+  /* allocate memory for Jinvgrad */
+  TRY_MALLOC( Jinvgrad, dim*bas_n, double, assem_poisson_e1);
+
+  /* clear K */
+  sparse_empty(K);
+  /* clear rhs, P */
+  for (i=0; i<vx_nr; i++)
+    {
+      (*rhs).V[i]=0.0;
+      (*P).V[i]=0;
+    }
+
+  /* loop over all elements */
+  for (el=0; el<(*m).el_nr; el++)
+    {
+      /* set elK to zero */
+      for (i=0; i<bas_n*bas_n; i++) 
+	{
+	  elK[i]=0.0;
+	}
 
 
+      /* Jacobian is constant for linear elements, compute and invert
+	 only once!*/
+      {
+	k=0;
+	  /* compute x_k and the Jacobian at this point */
+	  /* x_k=0 */
+	  for (i=0; i<dim; i++)
+	    x_k[i]=0.0;
+	  /* Jac=0 */
+	  for (i=0; i<dim*dim; i++)
+	    Jac[i]=0.0;
+	  
+	  /* x_k = sum_{i=nodes} vertex(i)*phi_i,
+	     Jac = sum_{i=nodes} vertex(i)*gradphi_i^T */
+	  for (i=0;i<bas_n; i++)
+	    {
+	      for (j=0; j<dim; j++)
+		{
+		  x_k[j]+= 
+		    (*m).vertex[(*m).elem[el*el_w+MCE1ELNOD1+i]*vx_w
+				+MCE1VXSTRT+j]
+		    * phi[k*bas_n+i];
+		  for (r=0; r<dim; r++)
+		    {
+		      Jac[j*dim+r]+= 
+			(*m).vertex[(*m).elem[el*el_w+MCE1ELNOD1+i]*vx_w
+				    +MCE1VXSTRT+j]
+			* gradp[k*bas_n*dim +i*dim +r];
+		    }
+		}
+	    }
+	  /* print Jac: 
+	  printf("\nelement %d Jac:\n",(int) el); for (i=0; i<dim; i++) {
+	  printf("%7f %7f \n", Jac[i*dim+0], Jac[i*dim+1]); } */
+
+#ifdef FEINSLAPACKINV
+	  /* get detJac and Jacinv by computing triangular factors
+	     first */
+	  dgetrf_(&fdim, &fdim, Jac, &fdim, IPIV, &info);
+	  
+	  if(info != 0)
+	    {
+	      /* Jac is singular */
+	      fprintf(stderr, "assem_poisson_e1: Jacobian is singular? "
+		      "element %"dFIDX",  dgetrf returned info=%d\n", el, info);
+	      return FAIL;
+	    }
+
+	  /* now Jac = P*L*U,
+	     detJac  = det(P)*det(L)*det(U)
+             det(L)  = 1 */
+	  detJac=1.0;
+	  for (i=0; i<dim; i++) 
+	    {
+	      detJac *= Jac[i*dim+i];
+	      if (IPIV[i] != (i+1)) detJac = -detJac;
+	    }
+	  dgetri_( &fdim, Jac, &fdim, IPIV, WORK, &f_LWORK, &info );
+	  if(info != 0)
+	    {
+	      /* Jac is singular */
+	      fprintf(stderr, "assem_poisson_e1: error inverting "
+		      "triangular factors of Jacobian " 
+		      "element %"dFIDX",  dgetri returned info=%d\n", el, info);
+	      return FAIL;
+	    }
+#else
+	  err=matrix_3x3_invert(Jac,3,Jac,3,&detJac);
+	  FUNCTION_FAILURE_HANDLE( err, matrix_3x3_invert, assem_poisson_e1);
+#endif
+      }
+	  
+      /* loop over all integration points */
+      for (k=0; k<iform.num_points; k++)
+	{
+
+	  /* Jinvgrad= Jacinv * gradphi[k,:,:]  */
+	  dgemm_( &fNoTrans, &fNoTrans, &fdim, &fbas_n, &fdim, 
+		  &done, Jac, &fdim, &(gradp[k*bas_n*dim]), &fdim,
+		  &dzero, Jinvgrad, &fdim );
+
+	  
+	  dhelp=iform.weights[k]*fabs(detJac);
+	  /* elK += |detJac|*weigth[k] * Jinvgrad^T*Jinvgrad */
+	  dgemm_( &fTrans, &fNoTrans, &fbas_n, &fbas_n, &fdim,
+		  &dhelp, Jinvgrad, &fdim, Jinvgrad, &fdim,
+		  &done, elK, &fbas_n );
+
+	  /* acumulate rhs */
+	  //	  if ((*m).vols[el*vo_w+MCE1VORHSF]>=0)
+	  {
+	    /* rhs_i = int (f*phi_i) 
+	       = sum weight_k*|detJac| * f(x_k) * phi(x_k) */
+	    /* get f(x_k) */
+	    FIDX fun=(*m).vols[el*vo_w+MCE1VORHSF];
+	   
+	    if (fun!=-1)
+	      {
+		err=mesh_func_eval ( m, fun,
+				     x_k, 0.0,
+				     1, &fx_k, NULL, NULL);
+		FUNCTION_FAILURE_HANDLE(err,mesh_func_eval,
+					assem_poisson_e1);
+		
+		for (i=0; i<bas_n; i++)
+		  (*rhs).V[(*m).elem[el*el_w+i]]+=
+		    iform.weights[k]*fabs(detJac)*fx_k*phi[k*bas_n+i];
+	      }
+	  }
+
+	    //meshvol+=iform.weights[k]*fabs(detJac);
+	} /* end loop over all integration points */
+      /* print elK:
+	 printf("\nelement %d:\n",(int) el); for (i=0; i<bas_n; i++) {
+	 printf("%3d %7f %7f %7f\n",(int)(*m).elem[el*el_w+MCE1ELNOD1+i], elK[i*bas_n+0],
+	 elK[i*bas_n+1], elK[i*bas_n+2]); } /* */
+      
+
+      /* elK is ready, add to K */
+      err=sparse_add_local(K, NoTrans,
+			   bas_n, &((*m).elem[el*el_w+MCE1ELNOD1]),
+			   bas_n, &((*m).elem[el*el_w+MCE1ELNOD1]),
+			   elK, bas_n );
+      FUNCTION_FAILURE_HANDLE( err, sparse_add_local, assem_poisson_e1);
+	       
+    }
+  /* end loop over all elements */
+
+#warning poisson_e1: Neumann BC not accounted for in rhs yet!
+  /* first Neumann BC, otherwise they would collide with Dirichlet BC
+  */
+
+
+  /* loop over boundary, Dirichlet BC implementation */
+  for (bd=0; bd<(*m).bd_nr; bd++)
+    {
+      /* take Dirichlet BC into account */
+      if ((*m).bound[bd*bd_w + MCE1BDTYPE]==1)
+	{
+	  FIDX fci, nnodes, node[3], fu, err1, err2;
+	  
+	  /* check out which nodes */
+	  fci   = (*m).bound[bd*bd_w + MCE1BDFACE];
+	  nnodes=0;
+	  for (i=0; i<3; i++)
+	    {
+	      /* check unknown nodes in edge i */
+	      FIDX edi =(*m).face[fci*fc_w + MCE1FCEDG1+i];
+	      for (k=0; k<2; k++)
+		{
+		  int found=0;
+		  FIDX nodek=(*m).edge[edi*eg_w + MCE1EGNOD1+k];
+		  for (j=0; j<nnodes; j++)
+		    {
+		      if (node[j] == nodek)
+			{
+			  found=1;
+			}
+		    }
+		  if (found==0)
+		    {
+#ifdef DEBUGFEINS
+		      if (nnodes==3)
+			{
+			  fprintf(stderr, "assem_poisson_e1: face with "
+				  "more than 3 nodes?\n");
+			  return FAIL;
+			}
+#endif 
+		      node[nnodes]=nodek;
+		      nnodes++;
+		    }
+		}
+	    }
+#ifdef DEBUGFEINS
+	  if (nnodes<3)
+	    {
+	      fprintf(stderr, "assem_poisson_e1: face with "
+		      "less than 3 nodes? fci=%"dFIDX", nnodes=%"dFIDX"\n",
+		      fci, nnodes);
+	      return FAIL;
+	    }
+#endif 
+
+	  for (i=0; i<3; i++)
+	    {
+	      fu=(*m).bound[bd*bd_w + MCE1BDFNCT];
+
+	      err=mesh_func_eval(m, fu,
+				 &(*m).vertex[node[i]*vx_w+MCE1VXSTRT],
+				 0.0,
+				 1, &(*u0).V[node[i]],
+				 NULL, NULL);
+	      FUNCTION_FAILURE_HANDLE( err, mesh_func_eval, assem_poisson_e1);
+
+	      /* at this stage the projector array holds the info if a DOF
+		 is Dirichlet
+	      */
+	      (*P).V[node[i]]=1;
+	    }
+
+	} /* end this BD is Dirichlet */
+      
+    } /* end take Dirichlet BC into account */
+  /* now correct the projector to a list of Dirichlet DOFs */
+  j=0;
+  for (i=0; i<vx_nr; i++)
+    {
+      if ((*P).V[i]==1)
+	{
+	  /* this is a Dirichlet node, append it to the list */
+	  (*P).V[j]=i;
+	  j++;
+	}
+    }
+  (*P).len=j;
+
+  if ( (*K).type == SP_TYPE_FLEX )
+    {
+      err = sparse_convert_compressed_row( K );
+      FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row, 
+			       assem_poisson_e1 );
+    }
+
+  /* free local data */
+  free(Jinvgrad);
+  free(WORK);
+  free(IPIV);
+  free(Jac);
+  free(elK);
+
+  free_intdata (&iform);
+
+  //printf("assem_poisson_e1: meshvol=%20.15e\n", meshvol);
+
+  return SUCCESS;
+}
 
 
 
