@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -44,14 +44,14 @@ assembly.h
 #include "sparse_struct.h"
 
 
-int assem_poison_t1(struct sparse *K, struct vector *rhs,
+int assem_poisson_t1(struct sparse *K, struct vector *rhs,
 		    struct vector *u0, struct projector1 *P, 
 		    struct mesh *m
 /* performs the assembly of the stiffness matrix K and the right hand
    side vector rhs which result from finite element discretisation of
-   the Poison equation on the mesh m, such that 
+   the Poisson equation on the mesh m, such that 
        K u = rhs
-   defines the approximate solution u of the Poison equation
+   defines the approximate solution u of the Poisson equation
        Laplace(u) = f
    with boundary conditions as given in the mesh m    
    (only p1 triangles for now)
@@ -60,25 +60,26 @@ int assem_poison_t1(struct sparse *K, struct vector *rhs,
 
    Output: K       - stiffness matrix
            rhs     - righthand side vector
-	   u0      - initial guess for the solution of the equation
- 	             system which satisfies the Dirichlet BC
 	   P       - data for a projector which projects onto the
            	     linear subspace which doesn't change the values
            	     of DOFs related to Dirichlet BC
 
+   In/Out: u0      - initial guess for the solution of the equation
+ 	             system, modified to satisfy the Dirichlet BC
+
    Return: SUCCESS - success
            FAIL    - failure, see error message, output will not be
                      valid
 */);
 
-int assem_poison_t2(struct sparse *K, struct vector *rhs,
+int assem_poisson_t2(struct sparse *K, struct vector *rhs,
 		    struct vector *u0, struct projector1 *P,
 		    struct mesh *m
 /* performs the assembly of the stiffness matrix K and the right hand
    side vector rhs which result from finite element discretisation of
-   the Poison equation on the mesh m, such that 
+   the Poisson equation on the mesh m, such that 
        K u = rhs
-   defines the approximate solution u of the Poison equation
+   defines the approximate solution u of the Poisson equation
        Laplace(u) = f
    with boundary conditions as given in the mesh m    
    (p2 triangles)
@@ -87,21 +88,22 @@ int assem_poison_t2(struct sparse *K, struct vector *rhs,
 
    Output: K       - stiffness matrix
            rhs     - righthand side vector
-	   u0      - initial guess for the solution of the equation
- 	             system which satisfies the Dirichlet BC
 	   P       - data for a projector which projects onto the
            	     linear subspace which doesn't change the values
            	     of DOFs related to Dirichlet BC
 
+   In/Out: u0      - initial guess for the solution of the equation
+ 	             system, modified to satisfy the Dirichlet BC
+
    Return: SUCCESS - success
            FAIL    - failure, see error message, output will not be
                      valid
 */);
 
-int assem_poison_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
+int assem_poisson_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
 		       struct mesh *m
 /* performs the assembly of the stiffness matrix K which results from
-   finite element discretisation of the Poison equation with linear
+   finite element discretisation of the Poisson equation with linear
    triangle elements (T1) on the (T2) mesh m
 
    Input:  m       - the mesh
@@ -120,6 +122,34 @@ int assem_poison_t1_t2(struct sparse *K, FIDX vx_nr1, FIDX *dofs,
                      valid
 */);
 
+int assem_poisson_e1(struct sparse *K, struct vector *rhs,
+		    struct vector *u0, struct projector1 *P, 
+		    struct mesh *m
+/* performs the assembly of the stiffness matrix K and the right hand
+   side vector rhs which result from finite element discretisation of
+   the Poisson equation on the mesh m, such that 
+       K u = rhs
+   defines the approximate solution u of the Poisson equation
+       Laplace(u) = f
+   with boundary conditions as given in the mesh m    
+   (only p1 tetrahedra for now)
+
+   Input:  m       - the mesh
+
+   Output: K       - stiffness matrix
+           rhs     - righthand side vector
+	   P       - data for a projector which projects onto the
+           	     linear subspace which doesn't change the values
+           	     of DOFs related to Dirichlet BC
+
+   In/Out: u0      - initial guess for the solution of the equation
+ 	             system, modified to satisfy the Dirichlet BC
+
+   Return: SUCCESS - success
+           FAIL    - failure, see error message, output will not be
+                     valid
+*/);
+
 int assem_mass_tx(struct sparse *M,  
 		  struct mesh *m,
 		  FIDX type
