@@ -1,3 +1,36 @@
+/***********************************************************************
+
+    This file is part of
+    FEINS, Finite Element Incompressible Navier-Stokes solver,
+    which is expanding to a more general FEM solver and toolbox,
+    Copyright (C) 2003--2013, Rene Schneider 
+    <rene.schneider@mathematik.tu-chemnitz.de>
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program. If not, see <http://www.gnu.org/licenses/>.
+
+    Minor contributions to this program (for example bug-fixes and
+    minor extensions) by third parties automatically transfer the
+    copyright to the general author of FEINS, to maintain the
+    possibility of commercial re-licensing. If you contribute but wish
+    to keep the copyright of your contribution, make that clear in
+    your contribution!
+
+    Non-GPL licenses to this program are available upon request from
+    the author.
+
+************************************************************************/
+
 /* this file contains the definitions of the data structures */
 
 /* don't do anything if allready included */
@@ -188,6 +221,15 @@ struct mesh {
 			 physical constants, and scaling shape parameters
 			 related to the problem */
 
+  int meshgen;        /* the mesh generator involved in the creation
+			 of the mesh,
+			 ==0: no mesh generator;
+			 ==1: triangle (2d);
+			 ==2: gmsh (2d + 3d);
+		      */
+  void *meshgenDATA;  /* pointer to data required by the meshgen
+			 wrapper routines (e.g. to project new nodes
+			 to the geometry */
 
   /* detailed values for the positions of objects within the mesh
      arrays are element type dependent and therefore defined with text
@@ -245,13 +287,12 @@ struct multilvl {
         __out__=mlvlnode_ + mlvld_* (__mlvl__).levlvx[__l__] \
                 +(__mlvl__).nlevl[(__l__)+1]; \
      } \
-   else \
+   else	\
      { \
         __out__=-1; \
      } \
    }
 
-
 /* data structure for the BPX preconditioner */
 struct bpxdata
 {
@@ -474,6 +515,9 @@ struct solver_settings
 				   before writing the solution
 				   increases with mesh refinement */
 
+  double instat_write_delta;    /* time step for writing to file,
+                                   overrides intat_timesteps if !=-1.0
+				*/
 
 
   FIDX write_mesh;	        /* What to write to file (optional):
