@@ -8,6 +8,11 @@ void dgemm_ ( char *transa, char *transb, int *m, int *n, int *k,
 	      double *beta, double *c, int *ldc );
 
 /* LAPACK stuff: */
+void dgetrf_ ( int *M, int *N, double *A, int *LDA,
+	       int *IPIV, int *INFO );
+
+void dgetri_ ( int *N, double *A, int *LDA, int *IPIV, 
+               double *WORK, int *LWORK, int *INFO );
 
 void dpbtrf_ ( char *UPLO, int *N, int *KD, double *AB, int *LDAB,
 	       int *INFO );
