@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -58,6 +58,7 @@ TO_HEADER:
 /* function prototypes */
 #include "gen_aux.h"
 
+
 /*FUNCTION*/
 int gen_proj_bpx_tx(void *notused, struct vector *in,
 		    void *arg3, struct vector *out
@@ -154,7 +155,7 @@ int gen_proj_bpx_tx(void *notused, struct vector *in,
   level_old=(*ml).lmax+1;
   for (i=hi_nr-1; i>=0; i--)
     {
-      level=(*m).hier[i*hi_w+l_mctxhilvl];
+      level=(*m).hier[i*hi_w+l_mctxhilvl]+1;
       if (level<=0) printf("error, level=%d\n", (int) level);
       if (level_old!=level)
 	{
@@ -253,7 +254,7 @@ int gen_proj_bpx_tx(void *notused, struct vector *in,
   level_old=0; 
   for (i=0; i<hi_nr; i++)
     {
-      level=(*m).hier[i*hi_w+l_mctxhilvl];
+      level=(*m).hier[i*hi_w+l_mctxhilvl]+1;
 
       if (level_old!=level)
 	{
@@ -469,11 +470,14 @@ int gen_proj_MG_tx(void *arg1, struct vector *in,
 	      FUNCTION_FAILURE_HANDLE( err, sparse_GS_sweep_fwd,
 				       gen_proj_MG_tx);
 		    
-	      /* do smooths Gauss-Seidel sweeps backward */
-	      err=sparse_GS_sweep_bwd( &Ks[lvl], &bi,
-				       &invdiagi, smooths, &xi);
-	      FUNCTION_FAILURE_HANDLE( err, sparse_GS_sweep_bwd,
-				       gen_proj_MG_tx);
+	      if (lvl==0)
+		{
+		  /* do smooths Gauss-Seidel sweeps backward */
+		  err=sparse_GS_sweep_bwd( &Ks[lvl], &bi,
+					   &invdiagi, smooths, &xi);
+		  FUNCTION_FAILURE_HANDLE( err, sparse_GS_sweep_bwd,
+					   gen_proj_MG_tx);/* */
+		}
 	    }
 
 	  
@@ -566,6 +570,9 @@ int gen_proj_MG_tx(void *arg1, struct vector *in,
   return SUCCESS;
 }
 
+/* inline functions */
+#include "assembly_inline_funs.h"
+
 /*FUNCTION*/
 int gen_zz_interpolate_gradient_tx(struct mesh *m,
 				   int type,
@@ -583,7 +590,7 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
    and a posteriori error estimators. Part 1. The recovery technique,
    Int. J. Numer. Methods Eng., 33, 1331-1364 (1992)
 
-   but for now it only a simplified version of this idea.
+   but for now it is only a simplified version of this idea.
 
    
    Input:  m       - the mesh, has to be T1 or T2 
@@ -614,6 +621,7 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
 
   FIDX dim, basn, bas_n, vx_nr, el_w, vx_w;
 
+  enum elemtype eltype;
 
   double points_t1[2*3]={ 0.0, 0.0,
 			  1.0, 0.0,
@@ -624,6 +632,10 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
 			  0.5, 0.0,
 			  0.5, 0.5,
 			  0.0, 0.5 };
+  double points_e1[4*4]={ 0.0, 0.0, 0.0,
+			  1.0, 0.0, 0.0,
+			  0.0, 1.0, 0.0,
+			  0.0, 0.0, 1.0 };
   double *points;
 
 
@@ -668,20 +680,37 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
       return FAIL;
     }
 
-
   switch (type)
     {
     case 1:
-      l_mctxVXSTRT = MCT1VXSTRT;
-      l_mctxELNOD1 = MCT1ELNOD1;
-      points = points_t1;
-      basn   = 3;
+      switch (dim)
+	{
+	case 2:
+	  l_mctxVXSTRT = MCT1VXSTRT;
+	  l_mctxELNOD1 = MCT1ELNOD1;
+	  points = points_t1;
+	  basn   = 3;
+	  eltype = tria;
+	  break;
+	case 3:
+	  l_mctxVXSTRT = MCE1VXSTRT;
+	  l_mctxELNOD1 = MCE1ELNOD1;
+	  points = points_e1;
+	  basn   = 4;
+	  eltype = tetra;
+	  break;
+	default:
+	  fprintf(stderr,"gen_zz_interpolate_gradient_tx: unknown dim=%d\n",
+		  dim);
+	  return FAIL;
+	}
       break;
     case 2:
       l_mctxVXSTRT = MCT2VXSTRT;
       l_mctxELNOD1 = MCT2ELNOD1;
       points = points_t2;
       basn   = 6;
+      eltype = tria;
       break;
     default:
       fprintf(stderr,"gen_zz_interpolate_gradient_tx: unknown type=%d\n",type);
@@ -690,7 +719,7 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
 
   /* init */
   /* initialise interpolation info  */
-  err= eval_basis( dim, tria, type, basn, points,
+  err= eval_basis( dim, eltype, type, basn, points,
 		   &bas_n, &phi, &gradp, &hessphi);
   FUNCTION_FAILURE_HANDLE( err, eval_basis, gen_zz_interpolate_gradient_tx);
   free( hessphi );
@@ -754,14 +783,10 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
 		  }
 	    }
 
-	  /* get detJac */
-	  detJac=Jac[0]*Jac[3]-Jac[2]*Jac[1];
-	  
-	  /* get Jacinv (here direct) */
-	  Jacinv[0]=1.0/detJac*Jac[3];
-	  Jacinv[1]=-1.0/detJac*Jac[1];
-	  Jacinv[2]=-1.0/detJac*Jac[2];
-	  Jacinv[3]=1.0/detJac*Jac[0];
+	  /* get detJac and Jacinv */
+	  err=matrix_dim_x_dim_invert(dim, Jac,dim, Jacinv,dim, &detJac);
+	  FUNCTION_FAILURE_HANDLE( err, matrix_dim_x_dim_invert,
+				   gen_zz_interpolate_gradient_tx);
 
 	  AdetJac = fabs(detJac);
 
@@ -909,25 +934,45 @@ int gen_error_est_ZZ_tx(struct mesh *m, FIDX ncomp, struct vector *uh,
 
   struct vector gradu_interp;
 
-  dim=2;
-  if ((*m).dim!=dim)
-    {
-      fprintf( stderr,
-	"gen_error_est_ZZ_tx is only implemented for 2D meshes!\n");
-      return FAIL;
-    }			    
+  enum elemtype eltype;
+
+  dim=(*m).dim;
 
   switch (type)
     {
     case 1:
-      l_mctxVXSTRT = MCT1VXSTRT;
-      l_mctxELNOD1 = MCT1ELNOD1;
-      order=4;
+      switch (dim)
+	{
+	case 2:
+	  l_mctxVXSTRT = MCT1VXSTRT;
+	  l_mctxELNOD1 = MCT1ELNOD1;
+	  eltype = tria;
+	  order=4;
+	  break;
+	case 3:
+	  l_mctxVXSTRT = MCE1VXSTRT;
+	  l_mctxELNOD1 = MCE1ELNOD1;
+	  eltype = tetra;
+	  order = 2;
+	  break;
+	default:
+	  fprintf( stderr,
+		   "gen_error_est_ZZ_tx: dim=%d is not supported\n",
+		   dim);
+	  return FAIL;
+	}
       break;
     case 2:
       l_mctxVXSTRT = MCT2VXSTRT;
       l_mctxELNOD1 = MCT2ELNOD1;
+      eltype = tria;
       order=7;
+      if (dim!=2)
+	{
+	  fprintf( stderr,
+		   "gen_error_est_ZZ_tx type=2 is only implemented for 2D meshes!\n");
+	  return FAIL;
+	}			    
       break;
     default:
       fprintf(stderr,"gen_error_est_ZZ_tx: unknown type=%d\n",type);
@@ -945,7 +990,7 @@ int gen_error_est_ZZ_tx(struct mesh *m, FIDX ncomp, struct vector *uh,
 
   subtypes=type;
 
-  err=cubature_bases( dim, order, tria, 1, &subtypes, &iform); 
+  err=cubature_bases( dim, order, eltype, 1, &subtypes, &iform); 
   FUNCTION_FAILURE_HANDLE( err, cubature_bases, gen_error_est_ZZ_tx);
 
   /* make phi and gradphi better accessible */
@@ -1004,14 +1049,11 @@ int gen_error_est_ZZ_tx(struct mesh *m, FIDX ncomp, struct vector *uh,
 		    *gradp[k*bas_n*dim+i*dim +r];
 		}
 
-	  /* get detJac */
-	  detJac=Jac[0]*Jac[3]-Jac[2]*Jac[1];
-	  
-	  /* get Jacinv (here direct) */
-	  Jacinv[0]=1.0/detJac*Jac[3];
-	  Jacinv[1]=-1.0/detJac*Jac[1];
-	  Jacinv[2]=-1.0/detJac*Jac[2];
-	  Jacinv[3]=1.0/detJac*Jac[0];
+	  /* get detJac and Jacinv */
+	  err=matrix_dim_x_dim_invert(dim, Jac,dim, Jacinv,dim, &detJac);
+	  FUNCTION_FAILURE_HANDLE( err, matrix_dim_x_dim_invert,
+				   gen_zz_interpolate_gradient_tx);
+
 	  AdetJac = fabs(detJac);
 	
 	  /* real world gradient, not constant for quadratic elements */
@@ -1344,7 +1386,7 @@ void heapSort(double *a, int *b, int len
     {     
       /* sift down the node at index start to the proper place such 
 	 that all nodes below the start index are in heap order */
-      siftDown(a, b, start, len-1);
+      heapSort_siftDown(a, b, start, len-1);
       start--;
       /* after sifting down the root all nodes/elements are in heap order */
     }
@@ -1354,9 +1396,9 @@ void heapSort(double *a, int *b, int len
     {
       /* swap the root(maximum value) of the heap with the last
 	 element of the heap */
-      swap(a, b, end, 0);
+      heapSort_swap(a, b, end, 0);
       /* put the heap back in max-heap order */
-      siftDown(a, b, 0, end-1);
+      heapSort_siftDown(a, b, 0, end-1);
       /* decrease the size of the heap by one so that the previous 
 	 max value will stay in its proper placement */
       end--;
@@ -1364,7 +1406,7 @@ void heapSort(double *a, int *b, int len
 }
   
 /*FUNCTION*/
-void siftDown(double *a, int *b, int start, int end
+void heapSort_siftDown(double *a, int *b, int start, int end
 /* part of the Heap Sort algorithm
 
   INPUT/OUTPUT: a   - an unordered array a of length count
@@ -1389,7 +1431,7 @@ void siftDown(double *a, int *b, int start, int end
 	}
       if (a[root]<a[child]) /* out of max-heap order */
 	{
-	  swap(a, b, root, child);
+	  heapSort_swap(a, b, root, child);
 	  root = child; /* repeat to continue sifting down the child now */
 	}
       else return;
@@ -1398,7 +1440,7 @@ void siftDown(double *a, int *b, int start, int end
 }
   
 /*FUNCTION*/
-void swap(double *a, int *b, int pos1, int pos2
+void heapSort_swap(double *a, int *b, int pos1, int pos2
 /* swaps the two entries pos1 and pos2 in the array a and b
 
   INPUT/OUTPUT : a     - an unordered array a of length count
