@@ -1,3 +1,36 @@
+/***********************************************************************
+
+    This file is part of
+    FEINS, Finite Element Incompressible Navier-Stokes solver,
+    which is expanding to a more general FEM solver and toolbox,
+    Copyright (C) 2003--2013, Rene Schneider 
+    <rene.schneider@mathematik.tu-chemnitz.de>
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program. If not, see <http://www.gnu.org/licenses/>.
+
+    Minor contributions to this program (for example bug-fixes and
+    minor extensions) by third parties automatically transfer the
+    copyright to the general author of FEINS, to maintain the
+    possibility of commercial re-licensing. If you contribute but wish
+    to keep the copyright of your contribution, make that clear in
+    your contribution!
+
+    Non-GPL licenses to this program are available upon request from
+    the author.
+
+************************************************************************/
+
 /*
 FILE lame_adj.c
 HEADER lame_adj.h
@@ -2333,6 +2366,38 @@ int lame_eppler_conjecture_2010_rem3_3( struct mesh *m,
   FUNCTION_FAILURE_HANDLE( err, lame_eppler_conjecture_2010_rem3_3,
 			   lame_eppler_conjecture_2010_rem3_3);
 
+
+  /*
+  {
+    struct vector cheat_sigma;
+    cheat_sigma.len=dim*vx_nr;
+
+    cheat_sigma.V=&sigmu.V[0];
+    err=mesh_write_solution_vtk_t2( m, &cheat_sigma, dim, NULL, 0,
+				    25, "visual/eppler_sigma_u__1" );
+    FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+			     lame_eppler_conjecture_2010_rem3_3);
+
+    cheat_sigma.V=&sigmu.V[dim*vx_nr];
+    err=mesh_write_solution_vtk_t2( m, &cheat_sigma, dim, NULL, 0,
+				    25, "visual/eppler_sigma_u__2" );
+    FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+			     lame_eppler_conjecture_2010_rem3_3);
+
+
+    cheat_sigma.V=&sig_gamma.V[0];
+    err=mesh_write_solution_vtk_t2( m, &cheat_sigma, dim, NULL, 0,
+				    31, "visual/eppler_sigma_gamma_u__1" );
+    FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+			     lame_eppler_conjecture_2010_rem3_3);
+    cheat_sigma.V=&sig_gamma.V[dim*vx_nr];
+    err=mesh_write_solution_vtk_t2( m, &cheat_sigma, dim, NULL, 0,
+				    31, "visual/eppler_sigma_gamma_u__2" );
+    FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+			     lame_eppler_conjecture_2010_rem3_3);
+  }/*/
+  
+
   /*************************************************************
      build the bound_elem list
   **************************************************************/
