@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -791,13 +791,13 @@ int GMRES( int m, int re_max,
 		      = 1 : use it,
 		      = 0 : don't use it, in this case the zero vector
 		            is used instead
-	   multfunc - pointer to a function performing the matrix
+           multfunc - pointer to a function performing the matrix
                       vector multiplication Mat*vec, syntax:
 		      int multfunc(void *Mat, struct vector *vec, 
 		                   struct vector *out),
 		      where Mat is the pointer provided in the call to
-		      PCG and vec is a vector of the same type as the
-		      solution vector x, 
+		      this routine and vec is a vector of the same
+		      type as the solution vector x, 
 		      must return SUCCESS on success and FAIL in case
 		      of error 
 	   precondi - pointer to a function performing the
@@ -806,25 +806,27 @@ int GMRES( int m, int re_max,
 		      system matrix Mat, e.g. C^{-1} can be a
 		      cheap approximation of the inverse of Mat,
 		      the quality of the preconditioner has huge
-		      influence on the performance of PCG,
+		      influence on the performance of this solver,
 		      syntax:
 		      int precondi(void *Mat, struct vector *vec,
 		                   void *prec_dat, struct vector *out),
-		      where Mat as provided to PCG, vec a vector of
-		      same type as solution vector x, prec_dat as
-		      provided to PCG, must return SUCCESS on success
-		      and FAIL in case of error,
+		      where Mat as provided to this routine, vec a
+		      vector of same type as solution vector x,
+		      prec_dat as provided to this routine, must
+		      return SUCCESS on success and FAIL in case of
+		      error, 
 		      if precondi==NULL then the function is not
 		      called but C^{-1}=I is used (no preconditioning)
            Mat      - pointer to data necessary to perform the
                       multiplication with the matrix, 
 		      in general this should be some representation of
-		      the matrix itself, but it is not used in PCG
-		      directly, but only through multfunc and precondi
+		      the matrix itself, but it is not used in this
+		      routine directly, but only through multfunc and
+		      precondi 
 	   rhs      - the right-hand side vector of the equation system
 	   prec_dat - pointer to additional data for the
-	              preconditioner, not used in PCG itself, but only
-	              in precondi
+	              preconditioner, not used in this routine itself,
+	              but only in precondi
 
    In/Out: x        - on Input:  initial iterate, if use_initial_x==1,
                                  else not used as input,
@@ -1233,3 +1235,160 @@ int GMRES( int m, int re_max,
 
   return check;
 }
+
+
+
+/*FUNCTION*/
+int chebyshev_semi_iteration(  
+	   struct vector *x,
+	   int iter,
+	   int (*multfunc)(void *Mat, struct vector *vec,
+			   struct vector *out), 
+	   int (*precondi)(void *Mat, struct vector *vec,
+			   void *prec_dat, struct vector *out),
+	   void *Mat, struct vector *rhs, void *prec_dat,
+	   double lambdaMin, double lambdaMax
+/* Chebyshev semi iteration for solving systems of linear
+   equations, see ???
+   approximates a solution of 
+
+          Mat*x = rhs
+
+   Input:  iter     - the fixed number of iterations to be performed,
+           multfunc - pointer to a function performing the matrix
+                      vector multiplication Mat*vec, syntax:
+		      int multfunc(void *Mat, struct vector *vec, 
+		                   struct vector *out),
+		      where Mat is the pointer provided in the call to
+		      this routine and vec is a vector of the same
+		      type as the solution vector x, 
+		      must return SUCCESS on success and FAIL in case
+		      of error 
+	   precondi - pointer to a function performing the
+                      preconditioning operation out=C^{-1}*vec,
+		      ideally C should be spectral equivalent to the
+		      system matrix Mat, e.g. C^{-1} can be a
+		      cheap approximation of the inverse of Mat,
+		      the quality of the preconditioner has huge
+		      influence on the performance of this solver,
+		      syntax:
+		      int precondi(void *Mat, struct vector *vec,
+		                   void *prec_dat, struct vector *out),
+		      where Mat as provided to this routine, vec a
+		      vector of same type as solution vector x,
+		      prec_dat as provided to this routine, must
+		      return SUCCESS on success and FAIL in case of
+		      error, 
+		      if precondi==NULL then the function is not
+		      called but C^{-1}=I is used (no preconditioning)
+           Mat      - pointer to data necessary to perform the
+                      multiplication with the matrix, 
+		      in general this should be some representation of
+		      the matrix itself, but it is not used in this
+		      routine directly, but only through multfunc and
+		      precondi 
+	   rhs      - the right-hand side vector of the equation system
+	   prec_dat - pointer to additional data for the
+	              preconditioner, not used in this routine itself,
+	              but only in precondi
+	   lambdaMin- minimal eigenvalue of the preconditioned system
+	   lambdaMax- maximal eigenvalue of the preconditioned system
+
+   Output: x        - approximate solution of the system
+           
+   Return: SUCCESS  - stopping criterion is fulfilled
+           FAIL     - general failure, see error messages
+*/
+	   ){
+  struct vector res, *ynew, *ymid, *yold, *tmp, y1, y2, y3, z; 
+                             /* residual, and vectors for the
+				iteration */
+  double omega,rho,scale;
+
+  int it;
+  FIDX i, len;
+  int err;
+
+  len=(*x).len;
+
+  /* make sure a vector_free or free does not result in segfaults */
+  res.V=NULL;
+
+  err=vector_alloc(&z, len);
+  FUNCTION_FAILURE_HANDLE(err, vector_alloc, chebyshev_semi_iteration);
+
+  err=vector_alloc(&y1, len);
+  FUNCTION_FAILURE_HANDLE(err, vector_alloc, chebyshev_semi_iteration);
+  err=vector_alloc(&y2, len);
+  FUNCTION_FAILURE_HANDLE(err, vector_alloc, chebyshev_semi_iteration);
+  err=vector_alloc(&y3, len);
+  FUNCTION_FAILURE_HANDLE(err, vector_alloc, chebyshev_semi_iteration);
+
+  err=vector_alloc(&res, len);
+  FUNCTION_FAILURE_HANDLE(err, vector_alloc, chebyshev_semi_iteration);
+  
+  ynew=&y3;
+  ymid=&y2;
+  yold=&y1;
+
+  for (i=0; i<len; i++)
+    {
+      ymid->V[i] = 0.0;
+      yold->V[i] = 0.0;
+    }
+
+  omega = 1.0;
+      
+  rho = (lambdaMax-lambdaMin)/(lambdaMax+lambdaMin);
+  scale= 2.0/(lambdaMin+lambdaMax);
+
+  for (it=0; it < iter; it++)
+    {
+      omega = 1.0/(1.0-(omega*rho*rho)/4);
+
+      /* res = Mat*ymid */
+      err= (*multfunc)( Mat, ymid, &res);
+      FUNCTION_FAILURE_HANDLE(err, multfunc, chebyshev_semi_iteration);
+      /* res = rhs - Mat*ymid */
+      for (i=0; i<len; i++)
+	{
+	  res.V[i]=(*rhs).V[i]-res.V[i];
+	}
+
+      /* z = C^-1 * res; */ 
+      err= (*precondi)(Mat, &res, prec_dat, &z);
+      FUNCTION_FAILURE_HANDLE(err, precondi, chebyshev_semi_iteration);
+
+      /* ynew = omega*(scale*z+ymid-yold)+yold */
+      for (i=0; i<len; i++)
+	{
+	  ynew->V[i]=omega*(scale*z.V[i]+ymid->V[i]-yold->V[i])+yold->V[i];
+	}
+
+
+      /* yold = ymid; ymid = ynew; */
+      tmp=yold;
+      yold=ymid;
+      ymid=ynew;
+      ynew=tmp;
+
+    } /* end for it */
+
+
+  /* copy result to x */
+  for (i=0; i<len; i++)
+    {
+      x->V[i]=ymid->V[i];
+    }
+
+  /* clean up */
+  vector_free(&res);
+
+  vector_free(&y3);
+  vector_free(&y2);
+  vector_free(&y1);
+
+  vector_free(&z);
+
+  return SUCCESS;
+}
