@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -50,6 +50,14 @@ void mesh_init( struct mesh *m
                       array lengths etc. to zero
 */);
 
+void mesh_info( FILE *outf, struct mesh *m
+/* printf info on the mesh to file outf (e.g. outf=stdout),
+
+   Input:  m        - a mesh
+   Output: (to file outf)
+
+*/);
+
 int mesh_refine_uniform_t1( struct mesh *m 
 /* refines all elements of the mesh m, all necessarry new parts are
    created thereby, increases mesh.lvl
@@ -58,7 +66,7 @@ int mesh_refine_uniform_t1( struct mesh *m
                      elements, faces, edges, vertices, boundary
                      elements and hierarchy entries are created,
 		     the element el and the face el are thereby
-		     replaced by one of their childs, the edges
+		     replaced by one of their children, the edges
 		     remain to allow other elements to stay intact,
 		     if necessary the fields of the mesh are
 		     reallocated to have space for the new stuff
@@ -78,7 +86,7 @@ int mesh_refine_element_t1( struct mesh *m , FIDX el
                       elements, faces, edges, vertices, boundary
                       elements and hierarchy entries are created,
 		      the element el and the face el are thereby
-		      replaced by one of their childs, the edges
+		      replaced by one of their children, the edges
 		      remain to allow other elements to stay intact,
 		      if necessary the fields of the mesh are
 		      reallocated to have space for the new stuff
@@ -626,7 +634,7 @@ int mesh_read_compute_conns_t1( struct mesh *m
    In/Out:  m       - (given by reference) a mesh, the edge and face
                       parts are completed
 
-   retun:   SUCCESS - success,
+   return:  SUCCESS - success,
             FAIL    - failure, see error message 
 */			 );
 
@@ -638,7 +646,7 @@ int mesh_refine_uniform_t2( struct mesh *m
                       elements, faces, edges, vertices, boundary
                       elements and hierarchy entries are created,
 		      the element el and the face el are thereby
-		      replaced by one of their childs, the edges
+		      replaced by one of their children, the edges
 		      remain to allow other elements to stay intact,
 		      if necessary the fields of the mesh are
 		      reallocated to have space for the new stuff
@@ -658,7 +666,7 @@ int mesh_refine_element_t2( struct mesh *m , FIDX el
                       elements, faces, edges, vertices, boundary
                       elements and hierarchy entries are created,
 		      the element el and the face el are thereby
-		      replaced by one of their childs, the edges
+		      replaced by one of their children, the edges
 		      remain to allow other elements to stay intact,
 		      if necessary the fields of the mesh are
 		      reallocated to have space for the new stuff
@@ -704,6 +712,18 @@ int mesh_t2_get_elem_from_face( struct mesh *m , FIDX i
                      valid
 */);
 
+int mesh_reorder_tx( struct mesh *m, int type
+/* reorders the vertices, faces, elements, such that higher cache
+   efficiency can be achied
+   
+   Input:  type    - the type of the mesh, type=1 linear, =2 quadratic
+
+   In/Out: m       - the mesh, node numbers are overwritten
+
+   Return: SUCCESS - success,
+           FAIL    - failure, see error message
+*/);
+
 void multilvl_free( struct multilvl *ml
 /* empties the multilvl data structure, such that it can be freed
 
@@ -796,8 +816,9 @@ int mg_init_tx( struct sparse *Ks, struct mesh *m, struct multilvl *ml,
    
    Input:  Ks      - array of sparse matrices, K[i] is the stiffness
                      matrix for the mesh on the i-th level
-           m       - the mesh (T2) to which the mgdata shall correspond
-                     (the important bit is the element hierarchy data)
+           m       - the mesh to which the mgdata shall correspond
+                     (the important bits are the hierarchy and element
+		     hierarchy data)
 	   ml      - multilevel data
 	   P       - Projector for boundary conditions, give P=NULL if
 	             not wanted
@@ -855,13 +876,17 @@ int mesh_refine_green_element_t2( struct mesh *m , FIDX el
    refinement, all necessarry new parts are
    created thereby
 
+   method according to E. B\"ansch, Local Mesh Refinement in 2 and 3
+   Dimensions, Impact of Computing in Science and Engineering, Vol. 3,
+   pp. 181--191, 1991
+
    Input:  el       - the element/face to be refined
 
    In/Out: m        - the mesh, is modified accordingly, e.g. new
                       elements, faces, edges, vertices, boundary
                       elements and hierarchy entries are created,
 		      the element el and the face el are thereby
-		      replaced by one of their childs, the edges
+		      replaced by one of their children, the edges
 		      remain to allow other elements to stay intact,
 		      if necessary the fields of the mesh are
 		      reallocated to have space for the new stuff
@@ -874,13 +899,18 @@ int mesh_refine_green_element_t2( struct mesh *m , FIDX el
 int mesh_refine_adaptive_t2( struct mesh *m, FIDX *marked_elem
 /* refines all marked elements of the mesh m with the
    Baensch-Green refinement, all necessarry new parts are
-   created thereby
+   created thereby,
+
+   method according to E. B\"ansch, Local Mesh Refinement in 2 and 3
+   Dimensions, Impact of Computing in Science and Engineering, Vol. 3,
+   pp. 181--191, 1991
+
 
    In/Out: m           - the mesh, is modified accordingly, e.g. new
                          elements, faces, edges, vertices, boundary
                          elements and hierarchy entries are created,
   		         the element el and the face el are thereby
-		         replaced by one of their childs, the edges
+		         replaced by one of their children, the edges
 		         remain to allow other elements to stay intact,
 		         if necessary the fields of the mesh are
 		         reallocated to have space for the new stuff
@@ -1248,6 +1278,19 @@ int mesh_more_faces ( struct mesh *m , FIDX need
                      valid
 */);
 
+int mesh_more_volumes ( struct mesh *m , FIDX need
+/* increases the number of volumes that can be held in the mesh by
+   reallocating memory
+
+   Input:  need       - the number of additional volumes that are
+                        required
+   In/Out: m          - the mesh 
+
+   Return: SUCCESS - success
+           FAIL    - failure, see error message, output will not be
+                     valid
+*/);
+
 int mesh_more_elems ( struct mesh *m , FIDX need
 /* increases the number of elems that can be held in the mesh by
    reallocating memory
