@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -228,11 +228,13 @@ int mesh_deform_t1_t2(struct mesh *msh1, FIDX lvlm,
 	if (child>=0)
 	  {
 	    coarse_bc_nodes[n_c_bc]=child-ml.nlevl[0+1];
+	    /* fprintf(stderr,"DEBUG: coarse_bc_nodes[%3"dFIDX"]=%3"dFIDX"  dof=%3"dFIDX"\n",
+	       n_c_bc, coarse_bc_nodes[n_c_bc],dof); */
 	    n_c_bc++;
 	  }
       }
       
-    /* printf("n_c_bc=%d\n", (int) n_c_bc); /* */
+    printf("n_c_bc=%d\n", (int) n_c_bc); /* */
     /* sparse_mat_write_file( &Ks_linel[0], "visual/Klinel.txt"); /* */
 
     err=coarse_mat_set( &Ks_linel[0], n_c_bc, coarse_bc_nodes,
