@@ -1,3 +1,37 @@
+/***********************************************************************
+
+    This file is part of
+    FEINS, Finite Element Incompressible Navier-Stokes solver,
+    which is expanding to a more general FEM solver and toolbox,
+    Copyright (C) 2003--2013, Rene Schneider 
+    <rene.schneider@mathematik.tu-chemnitz.de>
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program. If not, see <http://www.gnu.org/licenses/>.
+
+    Minor contributions to this program (for example bug-fixes and
+    minor extensions) by third parties automatically transfer the
+    copyright to the general author of FEINS, to maintain the
+    possibility of commercial re-licensing. If you contribute but wish
+    to keep the copyright of your contribution, make that clear in
+    your contribution!
+
+    Non-GPL licenses to this program are available upon request from
+    the author.
+
+************************************************************************/
+
+
 /* this file contains the definitions of the positions of data objects
    within the arrays of the mesh data structure */
 
@@ -25,7 +59,7 @@
 	   P*     - Pyramids    (3d)
 
 	   e.g. 
-           T1, T2 - linea resp. quadratic triangles
+           T1, T2 - linear resp. quadratic triangles
 	   Q1, Q2 - bilinear resp. biquadratic quadrangles
 	   E1, E2 - linear resp. quadratic tetrahedra
 	   H1, H2 - trilinear resp. triquadratic hexahedra
@@ -512,4 +546,223 @@
 
 
 
+
+
+/********************************************************************/
+/********************************************************************/
+/********************************************************************/
+/*                                                                  */
+/*                                                                  */
+/*       Definitions for 3D                                         */
+/*                                                                  */
+/********************************************************************/
+/********************************************************************/
+/********************************************************************/
+/********************************************************************/
+
+
+
+
+
+
+/********************************************************************/
+/*                                                                  */
+/*       Definitions for 3X (valid, for all 3d meshes)              */
+/*                                                                  */
+/********************************************************************/
+
+/*   function */
+#define MC3XFUTYPE 0              /* type of the function described,
+				     <  100.0, for scalar valued,
+				     >= 100.0, for vector valued, then
+				               FUDAT3 and FUDAT2 are valid and
+				               used just as FUDAT1
+				     if type or (type-100)
+				     ==   0.0, constant function,
+           			          FUDAT1 holds the constant,
+				     ==   1.0, linear function,
+				          FUDAT1 is the begin of a
+				          vector of coefficients c[i]
+				          such that
+					  f(x,y,z)=c[0]x+c[1]y+c[2]z+c[3]
+				     ==   2.0, quadratic function,
+				          FUDAT1 is the begin of a
+				          vector of coefficients c[i]
+				          such that
+				          f(x,y,z)=c[0]x^2+c[1]y^2+c[2]z^2
+                                                   +c[3]xy+c[4]xz+c[5]yz
+                                                   +c[6]x+c[7]y+c[8]z+c[9]   		 
+                                     ==   10.0, compound function 1 / 
+				          distinguished at a hyperplane
+				          FUDAT1 is the begin of a
+				          vector parameters c[i]
+				          such that
+					  if (c[0]x+c[1]y+c[2]z+c[3])>=0
+					  f(x,y)= the value the
+					     function with id c[4] has,
+					  if (c[0]x+c[1]y+c[2]z+c[3])<0
+					   f(x,y)= the value the
+					     function with id c[5] has,
+					  note: the function pointed
+  					       to has to be non
+  					       vector type, since
+  					       this is done for the
+  					       components seperately
+				                                   */
+#define MC3XFUDAT1 (MC3XFUTYPE+1) /* begin of function data for first
+				     or only component */
+#define MC3XFUDATX 10              /* length of function parameter
+				     (data) blocks, 1 such block for
+				     each component of a vector valued
+				     function */
+#define MC3XFUDAT2 (MC3XFUDAT1+MC3XFUDATX) 
+                                  /* begin of function data for second
+				     component */
+#define MC3XFUDAT3 (MC3XFUDAT2+MC3XFUDATX)
+                                    /* begin of function data for third
+				     component */
+#define MC3XFULN   (MC3XFUDAT3+MC3XFUDATX)
+                                  /* length of one function entry */
+
+
+/*   parameter data */
+#define MC3XPANUPO 0              /* position of the scaling parameter
+				     for the flow within the para field
+				  */
+#define MC3XPALN   (MC3XPANUPO+1) /* length of the parameter field */
+
+/*   shape segment data */
+#define MC3XSGLN   (0)
+                                  /* length of the shape segment field */
+
+
+/*   shape parameter data */
+#define MC3XSPDATA 0              /* position of the data */
+#define MC3XSPLN   (MC3XSPDATA+1) /* length of the shape parameter field */
+
+
+
+/********************************************************************/
+/*                                                                  */
+/*       Definitions for E1 (linear tetrahedra)                      */
+/*                                                                  */
+/********************************************************************/
+
+/*   vertices */
+#define MCE1VXSTRT 0              /* begin of component values, there
+				     are 3 components */
+#define MCE1VXLN   (MCE1VXSTRT+3) /* length of one vertex entry */
+
+/*   elements */
+#define MCE1ELNOD1 0              /* first node of the element   */
+#define MCE1ELLN   (MCE1ELNOD1+4) /* length of one element entry */
+
+
+
+/*   edges */
+#define MCE1EGNOD1 0              /* first node of the edge, there are
+				     2 nodes */
+#define MCE1EGCHL1 (MCE1EGNOD1+2) /* child data, =-1 <=> no childs,
+				     other <=> they are childs (there
+				     are 2 childs) */
+
+#define MCE1EGGEO  (MCE1EGCHL1+2) /* index of geometry info, which is
+				     to be observed for this edge
+				     =-1 for none
+				  */
+#define MCE1EGLVL  (MCE1EGGEO+1)  /* refinement level this edge
+				     belongs to */
+#define MCE1EGLN   (MCE1EGLVL+1) /* length of one edge entry */
+
+
+
+
+
+/*   faces */
+#define MCE1FCEDG1 0              /* first edge of the face, there are
+				     3 edges */
+
+#define MCE1FCCHL1 (MCE1FCEDG1+3) /* child data, =-1 <=> no children,
+				     other <=> they are children (there
+				     are 4 children) */
+
+#define MCE1FCBND  (MCE1FCCHL1+4) /* is boundary? =-1 is not,
+				     other is the boundary data number
+				     for this face */
+#define MCE1FCGEO  (MCE1FCBND +1) /* index of geometry info, which is
+				     to be observed for this edge
+				     =-1 for none
+				  */
+#define MCE1FCLVL  (MCE1FCGEO +1) /* refinement level this edge
+				     belongs to */
+
+
+#define MCE1FCPCSU (MCE1FCLVL +1) /* is performance criterion surface?
+				     =-1 is not,
+				     other is the pcsurf data number
+				     for this edge */
+
+#define MCE1FCLN   (MCE1FCPCSU+1) /* length of one face entry */
+
+/*   volumes */
+#define MCE1VOFAC1 0             /*first face of the tetrahedra, there are 
+                                   4 faces*/
+
+
+#define MCE1VORHSF (MCE1VOFAC1+4) /* righthand side function, 
+				     if =-1 then rhs=0,
+				     else index into function field */ 
+#define MCE1VOFRED (MCE1VORHSF+1) /* for Baensch-green-refinement:
+				     ==1 if father volume was red */
+#define MCE1VOPCVL (MCE1VOFRED+1) /* corresponding performance
+				     criterion volume entry, 
+				     if =-1 then there is no pcvol
+				     associated, else index into pcvol
+				     field */  
+#define MCE1VOLN   (MCE1VOPCVL+1) /* length of one volume entry,
+				   */
+
+/*   edge hierarchy, (same as T1) */
+#define MCE1HICHLD 0              /* child node in this hierarchy
+				     entry */
+#define MCE1HIFAT1 (MCE1HICHLD+1) /* first father node in this
+				     hierarchy entry , there are 2
+				     fathers for each child */
+#define MCE1HILVL  (MCE1HIFAT1+2) /* refinement level this entry
+				     belongs to */
+#define MCE1HILN   (MCE1HILVL +1) /* length of one hierarchy entry */
+
+/*   element oriented hierarchy */
+#define MCE1EHLN   (0)            /* not used for linear elements, 
+				     length of one hierarchy entry */
+
+
+/*   boundary */
+#define MCE1BDFACE 0              /* corresponding face */
+#define MCE1BDTYPE (MCE1BDFACE+1) /* type of the boundary condition,
+				    -1= no BC (but shape segment),
+				     1= Dirichlet (e.g. noslip),
+				     2= Neumann,
+				     3= no stress
+				  */
+#define MCE1BDFNCT (MCE1BDTYPE+1) /* function for boundary condition 
+				     (if applicable), points into the
+				     function field */
+
+ 
+#define MCE1BDORIE (MCE1BDFNCT+1) /* orientation of the corresponding
+				     face, scalar= 1 or -1 such that
+				     ??????????????? yet to be defined !
+				     is a OUTward vector */
+
+#define MCE1BDSSEG (MCE1BDORIE+1) /* marks the shape segement entry
+				     of this edge, 
+				     ==-1 not variable segment,
+				     >=0  defines a pointer into sseg */
+#define MCE1BDLN   (MCE1BDSSEG+1) /* length of one boundary entry */
+
+
+
+
+
 #endif
