@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -48,6 +48,7 @@ TO_HEADER:
 #include "mesh.h"
 #include "sparse.h"
 #include "lin_solver.h"
+#include "linsolve_hypre_AMG.h"
 
 #include "navsto_struct.h"
 #include "stokes_aux.h"
@@ -288,9 +289,9 @@ int navsto_solver(struct mesh *msh1, FIDX lvl_max,
 	    { x_old.V[i]=x.V[i];}
 
 	  /* update nu */
-	  nu_akt=fmax(nu, maxPe*1.3*nu_akt*dnu);
+	  nu_akt=fmax(nu, maxPe*1.3*nu_akt*dnu); /* */
 	  /* #warning "higher Pe"
-	     nu_akt=fmax(nu, 0.125*maxPe*1.3*nu_akt*dnu); /* */
+	     nu_akt=fmax(nu, 0.25*maxPe*1.3*nu_akt*dnu); /* */
 	  if (msh1->para[MC2XPANUPO]!=nu_akt)
 	    {
 	      msh1->para[MC2XPANUPO]=nu_akt; 
@@ -407,10 +408,34 @@ int navsto_solver(struct mesh *msh1, FIDX lvl_max,
 	      K.mg->smooths=1;
 	      K.mg->stop_eps=0.0;
 
+
+#if ( FP_VELOCITY_SOLVER <3 )
 	      strcpy(solver, "GMRES_Fp");
+#else
+	      strcpy(solver, "GMRES_Fp_AMG");
+	      err=gen_hypre_AMG_setup(&K.Fs[K.lvl],
+				      K.bn_nr, K.nodes, dim, 
+				      &K.AMGdataFs, 0 );
+	      FUNCTION_FAILURE_HANDLE( err, gen_hypre_AMG_setup, navsto_solver);
+	      { FIDX ApNodes=0;
+	      err=gen_hypre_AMG_setup(&K.Ap,
+				      1, &ApNodes, 1, 
+				      &K.AMGdataAp, 0 );
+	      }
+	      FUNCTION_FAILURE_HANDLE( err, gen_hypre_AMG_setup, navsto_solver);
+#endif
+ 
+
 	      err=GMRES( m, 100, 2, 0.0, leps, 1, &x, &resi, &iter,
 			 navsto_matrix_tim_vec, navsto_projector_w_precon,
-			 &K, &rhs, NULL ); /* */
+			 &K, &rhs, NULL ); 
+
+#if (FP_VELOCITY_SOLVER>=3)
+	      err=gen_hypre_AMG_free(&K.AMGdataFs);
+	      FUNCTION_FAILURE_HANDLE( err, gen_hypre_AMG_free, main); 	
+	      err=gen_hypre_AMG_free(&K.AMGdataAp);
+	      FUNCTION_FAILURE_HANDLE( err, gen_hypre_AMG_free, main); 	
+#endif
 	    }
 	  else
 	    {
@@ -761,9 +786,37 @@ int navsto_solver(struct mesh *msh1, FIDX lvl_max,
 		  }
 	      }
 
+#if ( FP_VELOCITY_SOLVER >=3 )
+	      if ( (K.Fs[K.lvl]).type!=SP_TYPE_COMPROW) 
+		{
+		  err = sparse_convert_compressed_row( &K.Fs[K.lvl]);
+		  FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row,
+					   navsto_solver);      
+		}
+	      err=gen_hypre_AMG_setup(&K.Fs[K.lvl],
+				      K.bn_nr, K.nodes, dim, 
+				      &K.AMGdataFs, 0 );
+	      FUNCTION_FAILURE_HANDLE( err, gen_hypre_AMG_setup, navsto_solver);
+	      { FIDX ApNodes=0;
+	      err=gen_hypre_AMG_setup(&K.Ap,
+				      1, &ApNodes, 1, 
+				      &K.AMGdataAp, 0 );
+	      }
+	      FUNCTION_FAILURE_HANDLE( err, gen_hypre_AMG_setup, navsto_solver);
+#endif
+ 
+
 	      err=GMRES( m, 100, 2, 0.0, adjeps, 0, &Psi, &resi, &iter,
 			 navsto_matrix_tim_vec, navsto_projector_w_precon,
 			 &K, &dIdsol[tc], NULL ); /* */ 
+
+#if (FP_VELOCITY_SOLVER>=3)
+	      err=gen_hypre_AMG_free(&K.AMGdataFs);
+	      FUNCTION_FAILURE_HANDLE( err, gen_hypre_AMG_free, main); 	
+	      err=gen_hypre_AMG_free(&K.AMGdataAp);
+	      FUNCTION_FAILURE_HANDLE( err, gen_hypre_AMG_free, main); 	
+#endif
+
 #else
 	      /* old adjoint handling, special multiplication and
 		 preconditioner routines */
