@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -1750,7 +1750,55 @@ int sparse_convert_compressed_row(struct sparse *K
 }
 
 
+/*FUNCTION*/
+int sparse_color_dofs(struct sparse *K
+/* colors the dofs of a matrix to allow parallel GS-smoother
 
+   In/Out:  K      - sparse matrix, coloring data ist added
+
+   Return: SUCCESS - success,
+           FAIL    - failure, see error message
+*/
+				  ){
+  FIDX i, j, endrow, row_nr;
+
+  FIDX max_col, this_cols, this, check_if_colored;
+
+  FIDX *color;
+
+  struct ilist *new_cols_first, *new_cols_last;
+
+  row_nr=K->row_nr;
+
+  /* how to test this ????
+     if (row_nr!=K->col_max)
+     {
+     fprintf(stderr, "sprase_color_dofs: require row_nr==col_nr\n");
+     return FAIL;
+     } */ 
+  
+  TRY_MALLOC(color, row_nr, FIDX, sprase_color_dofs);
+
+  /* initialise color for each dof */
+  for (i=0; i<row_nr; i++)
+    { 
+      color[i]=-1;
+    }
+
+  /* initialise list to be empty */
+  new_cols_first =NULL;
+  new_cols_last  =NULL;
+
+  /* outer loop, check each node if it has already been colored */
+  for (check_if_colored=0; check_if_colored<row_nr; check_if_colored++)
+    {
+#ifdef FEINS_have_warning 
+#warning "sparse_color_dofs: not complete yet"
+#endif 
+    }
+
+  return SUCCESS;
+}
 
 
 
