@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -386,6 +386,15 @@ int sparse_convert_compressed_row(struct sparse *K
            FAIL    - failure, see error message
 */);
 
+int sparse_color_dofs(struct sparse *K
+/* colors the dofs of a matrix to allow parallel GS-smoother
+
+   In/Out:  K      - sparse matrix, coloring data ist added
+
+   Return: SUCCESS - success,
+           FAIL    - failure, see error message
+*/);
+
 int sparse_extract_invdiag(struct sparse *K, struct vector *Kinvdiag
 /* the inverse of the diagonal of K is stored in the vector Kinvdiag
 
