@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -281,7 +281,7 @@ int stokes_projector_part_bpx(void *arg1, struct vector *in,
   level_old=(*ml).lmax+1;
   for (i=hi_nr-1; i>=0; i--)
     {
-      level=(*m).hier[i*hi_w+MCT2HILVL];
+      level=(*m).hier[i*hi_w+MCT2HILVL]+1;
       if (level<=0) printf("error, level=%d\n", (int) level);
       if (level_old!=level)
 	{
@@ -369,7 +369,7 @@ int stokes_projector_part_bpx(void *arg1, struct vector *in,
   level_old=0; 
   for (i=0; i<hi_nr; i++)
     {
-      level=(*m).hier[i*hi_w+MCT2HILVL];
+      level=(*m).hier[i*hi_w+MCT2HILVL]+1;
 
       if (level_old!=level)
 	{
@@ -486,10 +486,12 @@ int stokes_projector_part_MG(void *arg1, struct vector *in,
 
   lmax=(*ml).lmax;
 
-  if (lmax != K->lvl)
+  if (lmax < K->lvl-1)
     {
       fprintf(stderr,
 	      "stokes_projector_part_MG: lvl info contradicts!\n");
+      fprintf(stderr,
+	      "    lmax=%d    K->lvl=%d!\n",lmax, K->lvl);
       return FAIL;
     }
 
@@ -755,7 +757,10 @@ int stokes_projector_w_precon(void *arg1, struct vector *in,
         
    C^-1=[I           ] [  I          ] [A^-1        ]  velocity space 1
         [  I         ]*[       I     ]*[     A^-1   ]  velocity space 2
-        [    nu*Mp^-1] ]-B_1 -B_2  I ] [           I]  pressure space
+        [    nu*Mp^-1] [-B_1 -B_2  I ] [           I]  pressure space
+                       ###############
+		       optional, only 
+                       in non-symmetric version
 
    where A is the stiffness matrix of the Laplace operator, nu is 1/Re
    and Mp is the pressure space mass matrix, 
@@ -907,6 +912,8 @@ int stokes_projector_w_precon(void *arg1, struct vector *in,
 
   /* P^T done, now C^{-1} */
 
+#ifdef STOKES_NONSYMETRIC_PRECON
+#warning "use nonsymetric stokes preconditioner"
   /* rhs=[B -I]*[out.vel; rsh] */
   for (j=0; j<dim; j++)
     {
@@ -925,26 +932,40 @@ int stokes_projector_w_precon(void *arg1, struct vector *in,
 	  rhs.V[(*K).pdof[i] ]= -rhs.V[(*K).pdof[i] ];
 	}
     }
-  
+#endif
+
   /* solve M out = rhs */
   xi.len=pvx_nr;
   xi.n_max=pvx_nr;
   xi.V=&(*out).V[dim*vx_nr];
+#ifndef STOKES_PRECO_CHEBYSHEV
   if ( K->innersteps[dim]<0 )
     {
       err=PCG( 10000, 2, 0.0, (*K).innereps, 0, &xi, &resi, &iter,
-	       sparse_mul_mat_vec, NULL, &(*K).M, &rhs, NULL);
+	       sparse_mul_mat_vec, vector_diag_scale, &(*K).M, &rhs, 
+	       &(*K).Minvdiag);
       FUNCTION_FAILURE_HANDLE( err, PCG, stokes_projector_w_precon);
       K->innersteps[dim]=iter;
     }
   else
     {
       err=PCG( K->innersteps[dim], 0, 0.0, 0.0, 0, &xi, &resi, &iter,
-	       sparse_mul_mat_vec, NULL, &(*K).M, &rhs, NULL);
+	       sparse_mul_mat_vec, vector_diag_scale, &(*K).M, &rhs, 
+	       &(*K).Minvdiag);
       if (err==10) err=SUCCESS;
       FUNCTION_FAILURE_HANDLE( err, PCG, stokes_projector_w_precon);
     }
   (*K).innercount[dim]+=iter;
+#else
+  iter=5;
+  err=chebyshev_semi_iteration(  
+	 &xi, iter, 
+	 sparse_mul_mat_vec, vector_diag_scale, 
+	 &(*K).M, &rhs,  &(*K).Minvdiag,
+	 0.5, 2.0);
+  FUNCTION_FAILURE_HANDLE( err, chebyshev_semi_iteration, stokes_projector_w_precon);
+  (*K).innercount[dim]+=iter;
+#endif
 
   /* C^{-1} done, now P */
   /* get the mean pressure */
@@ -1134,8 +1155,8 @@ int stokes_write_streamfunc_t2( struct mesh *m, struct vector *usol,
   err=projector1_alloc( &P, (*m).vx_nr );
   FUNCTION_FAILURE_HANDLE( err, projector1_alloc, stokes_write_streamfunc_t21);
 
-  err=stokes_ass_streamfunc_poison_t2( &K, &rhs, &strf, &P, m, usol );
-  FUNCTION_FAILURE_HANDLE( err, stokes_stream_func_poison_t2,
+  err=stokes_ass_streamfunc_poisson_t2( &K, &rhs, &strf, &P, m, usol );
+  FUNCTION_FAILURE_HANDLE( err, stokes_ass_streamfunc_poisson_t2,
 			   stokes_write_streamfunc_t21);
 
   err=PCG( 10000, 1, 1e-7, 0.0, 1, &strf, &resi, &iter, sparse_mul_mat_vec, 
