@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -878,7 +878,7 @@ int stokes_pdof_init_t21( struct mesh *m, struct navsto_matrix *K
 
 
 /*FUNCTION*/
-int stokes_ass_streamfunc_poison_t2(struct sparse *K, struct vector *rhs,
+int stokes_ass_streamfunc_poisson_t2(struct sparse *K, struct vector *rhs,
 				    struct vector *psi0, struct projector1 *P,
 				    struct mesh *m, struct vector *usol
 /* performs the assembly of the stiffness matrix K and the right hand
@@ -932,7 +932,7 @@ int stokes_ass_streamfunc_poison_t2(struct sparse *K, struct vector *rhs,
   if (((*m).dim!=2)||((*usol).len!=2*(*m).vx_nr))
     {
       fprintf( stderr,
-	       "stokes_ass_streamfunc_poison_t2: cubature_bases returned "
+	       "stokes_ass_streamfunc_poisson_t2: cubature_bases returned "
 	       "error!\n");
       return FAIL;
     }
@@ -946,7 +946,7 @@ int stokes_ass_streamfunc_poison_t2(struct sparse *K, struct vector *rhs,
   if (err!=SUCCESS)
     {
       fprintf( stderr,
-	       "stokes_ass_streamfunc_poison_t2: cubature_bases returned "
+	       "stokes_ass_streamfunc_poisson_t2: cubature_bases returned "
 	       "error!\n");
       return FAIL;
     }
@@ -969,12 +969,12 @@ int stokes_ass_streamfunc_poison_t2(struct sparse *K, struct vector *rhs,
   fu_w  = (*m).fu_w;
 
   /* allocate memory for elK */
-  TRY_MALLOC( elK, bas_n*bas_n, double, assem_poison_t1);
+  TRY_MALLOC( elK, bas_n*bas_n, double, stokes_ass_streamfunc_poisson_t2);
   /* allocate memory for Jac, Jacinf */
-  TRY_MALLOC( Jac, dim*dim, double, assem_poison_t1);
-  TRY_MALLOC( Jacinv, dim*dim, double, assem_poison_t1);
+  TRY_MALLOC( Jac, dim*dim, double, stokes_ass_streamfunc_poisson_t2);
+  TRY_MALLOC( Jacinv, dim*dim, double, stokes_ass_streamfunc_poisson_t2);
   /* allocate memory for Jinvgrad */
-  TRY_MALLOC( Jinvgrad, dim*bas_n, double, assem_poison_t1);
+  TRY_MALLOC( Jinvgrad, dim*bas_n, double, stokes_ass_streamfunc_poisson_t2);
 
   /* clear K */
   sparse_empty(K);
@@ -1057,7 +1057,7 @@ int stokes_ass_streamfunc_poison_t2(struct sparse *K, struct vector *rhs,
 			   bas_n, &((*m).elem[el*el_w+MCT2ELNOD1]),
 			   elK, bas_n );
       FUNCTION_FAILURE_HANDLE( err, sparse_add_local, 
-			       stokes_ass_streamfunc_poison_t2);
+			       stokes_ass_streamfunc_poisson_t2);
       
     }
   /* end loop over all elements */
@@ -1070,7 +1070,7 @@ int stokes_ass_streamfunc_poison_t2(struct sparse *K, struct vector *rhs,
   if (bd_nr<1)
     {
       fprintf(stderr,
-	      "stokes_ass_streamfunc_poison_t2: no boundary?\n");
+	      "stokes_ass_streamfunc_poisson_t2: no boundary?\n");
       return FAIL;
     }
 
@@ -1120,7 +1120,7 @@ int stokes_ass_streamfunc_poison_t2(struct sparse *K, struct vector *rhs,
 	}
       if (found==0)
 	{
-	  fprintf( stderr, "stokes_ass_streamfunc_poison_t2: "
+	  fprintf( stderr, "stokes_ass_streamfunc_poisson_t2: "
 		   "boundary not closed?\n"); 
 	  return FAIL;
 	}
