@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2010, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -69,7 +69,7 @@ int main(int argc, char *argv[])
   struct projector1 P;
   
   struct vector b;
-  struct vector xtrial1, xtrial2, *x, *xtrial;
+  struct vector xtrial1, xtrial2, *x, *xtrial, diff, xwrite;
 
   struct mesh msh1;
   struct mesh msh2;
@@ -126,14 +126,43 @@ int main(int argc, char *argv[])
   double mhat1 = 2.113248654051871e+00;
   double mhat2 = 1.000000000000000e+00;
   double mhat3 = 4.226497308103742e-01;  
+  
+  /* output at intermediate points, 
+     J.Lang, Adaptive Multilevel Solution of Nonlinear Parabolic PDE
+     Systems, Springer 2001, 
+     Table C.2, page 132
+     equation (V.16), page 55 
+  */
+  double mtheta1_1=+2.5358983848622453, mtheta1_2=-0.5358983848622453;
+  double mtheta2_1=-1.1547005383792515, mtheta2_2=+1.7320508075688773;
+  double mtheta3_1=+1.1547005383792515, mtheta3_2=-0.7320508075688773;
+
+
+ 
 
   struct vector Un1, Un2, Un3, bdot, sumU;
   double factor1, factor2;
 
-  double rho=0.9, alpha_low=0.1, alpha_high=10, atol=1e-7, rtol=1e-5, tol;
-  double time, tau_old, tau_new, err_norm_old, err_norm_new;
+  // atol was 1e-7
+  double rho=0.9, alpha_low=0.1, alpha_high=10, atol=1e-4, rtol=1e-5, tol;
+  double time, tau_old, tau_new, err_norm_old, err_norm_new, tau_write,
+    t_write, time_old;
   int not_done=1;
 
+#ifdef HAVE_OPENMP
+#pragma omp parallel
+  {
+#pragma omp master
+    {
+      printf("OpenMP num_threads=%d\n",omp_get_num_threads());
+      printf("  should be number of phyisical cores, use environment variable "
+	     " OMP_NUM_THREADS to modify,\n");
+      printf("  e.g. (bash shell): \"export OMP_NUM_THREADS=4\" \n");
+      printf("  e.g. (most other): \"set OMP_NUM_THREADS 4\" \n");
+    }
+  }
+#endif
+
   if (argc>1)
     {
       printf("meshfile: %s \n", argv[1]);
@@ -155,6 +184,9 @@ int main(int argc, char *argv[])
   Tstrt       = set.instat_Time0;
   Tend        = set.instat_Tend;
 
+  tau_write   = set.instat_write_delta;
+  t_write     = Tstrt;
+
   dim=msh1.dim;
 
   // eps=msh1.para[0];
@@ -182,17 +214,20 @@ int main(int argc, char *argv[])
   FUNCTION_FAILURE_HANDLE( err, mesh_t1_to_t2, main);
 
   /* local refinement in a band of the domain */
-  for (lvl=0; lvl<(3+2+2)*2; lvl++) 
+  for (lvl=0; lvl<0*(3+1+1)*2; lvl++) 
     {
       FIDX *marker;
       FIDX nrmarked;
-      double c_band[]={0.0, 1.0}, d_band=-1900.0, width_band=30.0;
+      // 2km x 2km:
+      // double c_band[]={0.0, 1.0}, d_band=-1900.0, width_band=30.0;
+      // case A:
+      double c_band[]={0.0, 1.0}, d_band=-9900.0, width_band=30.0;
 
       TRY_MALLOC( marker, msh2.el_nr, FIDX, main);
 
-      if (lvl>=4) width_band=20;
-      if (lvl>=8) width_band=10;
-      if (lvl>=10) width_band=5;
+      if (lvl>=2) width_band=20;
+      if (lvl>=4) width_band=10;
+      if (lvl>=8) width_band=5;
 
       err=gen_band_marker_tx(&msh2, marker, &nrmarked,
 			     c_band, d_band, width_band, 2); 
@@ -215,7 +250,7 @@ int main(int argc, char *argv[])
 
 
   /* refine */
-  for (lvl=0; lvl<lvlm; lvl++)
+  for (lvl=0; lvl<lvlm-1; lvl++)
     {
       err=mesh_refine_uniform_t2( &msh2 );
       FUNCTION_FAILURE_HANDLE( err, mesh_refine_uniform_t2, main);
@@ -223,6 +258,40 @@ int main(int argc, char *argv[])
       tau         /= 2;
       write_steps *= 2;
     }
+  
+  /* local refinement in a band of the domain */
+  for (lvl=0; lvl<(3+1+1)*2-2; lvl++) 
+    {
+      FIDX *marker;
+      FIDX nrmarked;
+      // 2km x 2km:
+      // double c_band[]={0.0, 1.0}, d_band=-1900.0, width_band=30.0;
+      // case A:
+      double c_band[]={0.0, 1.0}, d_band=-9900.0, width_band=30.0;
+
+      TRY_MALLOC( marker, msh2.el_nr, FIDX, main);
+
+      if (lvl>=2) width_band=20;
+      if (lvl>=4) width_band=10;
+      if (lvl>=8) width_band=5;
+
+      err=gen_band_marker_tx(&msh2, marker, &nrmarked,
+			     c_band, d_band, width_band, 2); 
+      FUNCTION_FAILURE_HANDLE( err, gen_band_marker_tx, main);
+
+      if (nrmarked>0)
+	{
+	  /* refine the mesh */
+	  err=mesh_refine_adaptive_t2( &msh2, marker);
+	  FUNCTION_FAILURE_HANDLE( err, mesh_refine_uniform_t2, main);
+	}
+      printf("refined %6"dFIDX" elements, vx_nr=%6"dFIDX"\n",
+	     nrmarked,msh2.vx_nr);
+	      
+      
+      free(marker);
+    }
+
 
 
   vx_nr=msh2.vx_nr; 
@@ -241,6 +310,8 @@ int main(int argc, char *argv[])
   xtrial=&xtrial2;
   err=vector_alloc( x, vx_nr );
   err=vector_alloc( xtrial, vx_nr );
+  err=vector_alloc( &diff, vx_nr );
+  err=vector_alloc( &xwrite, vx_nr );
   err=vector_alloc( &b, vx_nr );
   err=vector_alloc( &bdot, vx_nr );
 
@@ -278,21 +349,31 @@ int main(int argc, char *argv[])
   tau_old  = 0.0;
 /* old time */
   time=Tstrt;
-  rtol=pow(tau,3.0)/(Tend-Tstrt);
+  time_old=time;
+
+  /* to this point the time step size was reduced like the space step
+     size, the error allowed in each time step is to be of the order
+     of the spatial discretisation error to achieve optimal combined
+     order, we use P2 elements, thus the L2 error should be h^3 */
+  rtol=pow(tau,3.0)/(Tend-Tstrt); 
+
+  /* atol=rtol; */
+  printf("atol=%8.2e   rtol=%8.2e\n", atol, rtol);
   for (tstep=0; not_done; tstep++)
     {
+      /* set all stage values to zero */
+      for (i=0; i<vx_nr; i++)
+	{
+	  Un1.V[i]= 0.0;
+	  Un2.V[i]= 0.0;
+	  Un3.V[i]= 0.0;
+	}
+
       /* for tstep==0 (time=Tstart) use the same output routine as
 	 during time advance,
 	 only start time advance at tstep==1 */
       if (tstep>0)
 	{
-	  /* set all stage values to zero */
-	  for (i=0; i<vx_nr; i++)
-	    {
-	      Un1.V[i]= 0.0;
-	      Un2.V[i]= 0.0;
-	      Un3.V[i]= 0.0;
-	    }
 
 	  /****************/
 	  /*              */
@@ -489,23 +570,30 @@ int main(int argc, char *argv[])
 	  TIMEGET;
 	  ts+=ti-t0;
 
-	  /* update x, compute norm(x) and norm(x-xhat) */
-	  norm=0.0;
-	  err_norm_new=0.0;
+	  /* update x, and compute diff=x-xhat */ 
 	  for (i=0; i<vx_nr; i++)
 	    {
-	      double diff, xnew;
+	      double diffi, xnew;
 	      
-	      diff   = m1*Un1.V[i] + m2*Un2.V[i] + m3*Un3.V[i];
+	      diffi   = m1*Un1.V[i] + m2*Un2.V[i] + m3*Un3.V[i];
 
-	      xnew   = (*x).V[i] + diff;
+
+	      xnew   = (*x).V[i] + diffi;
 
 	      (*xtrial).V[i] = xnew;
-	      norm += xnew*xnew;
 
-	      diff  -= mhat1*Un1.V[i] + mhat2*Un2.V[i] + mhat3*Un3.V[i];
-	      err_norm_new += diff*diff;
+	      diffi  -= mhat1*Un1.V[i] + mhat2*Un2.V[i] + mhat3*Un3.V[i];
+	      diff.V[i] = diffi;
 	    }
+	  /* compute norm_M(x) and norm_M(x-xhat) */
+	  norm=0.0;
+	  err=sparse_vec_mat_vec(&M, xtrial, xtrial, &norm);
+	  FUNCTION_FAILURE_HANDLE( err, sparse_vec_mat_vec, main);
+
+	  err_norm_new=0.0;
+	  err=sparse_vec_mat_vec(&M, &diff, &diff, &err_norm_new);
+	  FUNCTION_FAILURE_HANDLE( err, sparse_vec_mat_vec, main);
+
 
 	  printf("tstep=%5"dFIDX"  time=%8.6f  tau=%8.2e  |x|=%8.2e |x-xhat|/|x|=%8.2e   solver UMFPACK, size=%6"dFIDX", t_assem=%8.2e, t_mul=%8.2e, t_solver=%8.2e ", tstep, time, tau, sqrt(norm), sqrt(err_norm_new)/sqrt(norm), (*x).len, ta, tb, ti-t0);
 
@@ -540,6 +628,7 @@ int main(int argc, char *argv[])
 	      x=xtrial;
 	      xtrial=tmp;
 
+	      time_old=time;
 	      time += tau;
 
 	      tau_old=tau;
@@ -565,27 +654,51 @@ int main(int argc, char *argv[])
 	} /* end not first tstep */
 
 
-
-      write_steps=10;
-      if ((tstep%write_steps)==0)
+      while (time>=t_write)
 	{
-	  char fname[100];
-	  sprintf(fname, "visual/conv_diff_vtk_%05"dFIDX,
-		  (tstep/write_steps));
+	  double theta, mt1, mt2, mt3; 
+	  char fname[200];
+
+	  
+	  /*sprintf(fname, "visual/conv_diff_vtk_%05"dFIDX"_%12.6e",
+	    tstep,t_write);*/
+
+	  /* sprintf(fname, "visual/conv_diff_vtk_%07ld%07"dFIDX,
+	     lround((t_write-Tstrt)/tau_write),tstep); */
+	  sprintf(fname, "visual/conv_diff_vtk_%07ld",
+		  lround((t_write-Tstrt)/tau_write));
+
+	  /* interpolate the solution to intermediate time t_write */
+	  if (time-time_old>0)
+	    {
+	      theta=(t_write-time_old)/(time-time_old);
+	    }
+	  else
+	    {
+	      theta=0;
+	    }
+
+	  mt1=theta*(mtheta1_1+mtheta1_2*theta);
+	  mt2=theta*(mtheta2_1+mtheta2_2*theta);
+	  mt3=theta*(mtheta3_1+mtheta3_2*theta);
+
+	  for (i=0; i<vx_nr; i++)
+	    {
+	      /* xtrial was accepted, so it became x, at the same time
+		 xtrial was replaced by x(old) */
+	      
+	      xwrite.V[i] = (*xtrial).V[i] 
+		+ mt1*Un1.V[i] + mt2*Un2.V[i] + mt3*Un3.V[i];
+	    }
+	  
+
 	  /* write the solution for paraview visualisation */
-	  err=mesh_write_solution_vtk_t2( &msh2, NULL, 0, x, 1,
+	  err=mesh_write_solution_vtk_t2( &msh2, NULL, 0, &xwrite, 1,
 					  strlen(fname)+1,fname );
 	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t1,
 				   main); /* */
 
-
-	  sprintf(fname, "visual/conv_diff_xtrial_%05"dFIDX,
-		  (tstep/write_steps));
-	  /* write the solution for paraview visualisation */
-	  err=mesh_write_solution_vtk_t2( &msh2, NULL, 0, xtrial, 1,
-					  strlen(fname)+1,fname );
-	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t1,
-				   main); /* */
+	  t_write += tau_write;
 	}
 
 
@@ -613,6 +726,8 @@ int main(int argc, char *argv[])
 
   vector_free(&bdot);
   vector_free(&b);
+  vector_free(&xwrite);
+  vector_free(&diff);
   vector_free(xtrial);
   vector_free(x);
 
