@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -99,6 +99,10 @@ int main(int argc, char *argv[])
   char *buffer;
   FILE *out;
 
+  char orderstring[20];
+
+  strcpy(orderstring,"C");
+
   res_start = -1.0;
   stiffening = 1;  /* ==1 for real Lame', 
                       ==0 for stiffening of small elements */
@@ -163,7 +167,8 @@ int main(int argc, char *argv[])
   if (msh2.sg_nr>0)
     {
       err=mesh_deform_start_t2(&msh2, Ks);
-      FUNCTION_FAILURE_HANDLE( err, mesh_deform, main);
+#warning "errors ignored"
+      //FUNCTION_FAILURE_HANDLE( err, mesh_deform, main);
     }    
 
   vx_nr=msh2.vx_nr;     
@@ -235,6 +240,7 @@ int main(int argc, char *argv[])
 	  err=mg_init_tx( Ks, &msh2, &ml, &mg, &P);
 	  FUNCTION_FAILURE_HANDLE( err, mg_init_tx, main);
 	  
+
 	  /* generate the coarse grid matrix */
 	  n_c_bc=0;
 	  TRY_MALLOC(cmat, 1,  struct coarse_mat, main);
@@ -465,6 +471,39 @@ int main(int argc, char *argv[])
 
   if (stop==0)
     {
+#warning "mesh_reorder_tx for tests"
+      if (0==1)
+      {
+	/* write mesh to file */
+	struct vector bla;
+	
+	err=vector_alloc( &bla, dim*vx_nr );
+	FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
+	for (i=0; i<2*vx_nr; i++)
+	  bla.V[i]=0.0;
+
+	err=mesh_write_solution_vtk_t2( &msh2, &bla, dim, NULL, 0,
+					19, "visual/mesh_reorder" );
+	FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+				 main);
+
+	vector_free( &bla );
+
+	fprintf(stderr,"main: testing reordering of the mesh, switch to "
+		"solver=3 because all the multigrid stuff doesn't work "
+		"yet with the reordering\n"
+		"  !!! currently this breaks the derivative of the mesh "
+		"deformation !!!\n");
+	set.solver=3;
+
+	/* reorder */
+	err=mesh_reorder_tx(&msh2, 2);
+	FUNCTION_FAILURE_HANDLE( err, mesh_reorder_tx, main);
+
+	strcpy(orderstring,"front");
+
+      }
+
       /* reached finest level because max levels exhausted,
          need to solve once more */
       /* allocate memory for matrix Ks, P and the vectors */
@@ -486,6 +525,39 @@ int main(int argc, char *argv[])
 			    lambda, mu, stiffening, 2, 2);
       FUNCTION_FAILURE_HANDLE( err, assem_lame_tx, main);
 
+      /* write matrix to file */
+       if (0==1) /* WRITE MATRIX AND VECTORS*/
+	 {
+	   char name[100];
+	   
+	   /* is already in copressed row format */
+	   sprintf(name,"visual/Lame_Mat_%s_%07"dFIDX"_nodes.txt",
+		   orderstring, vx_nr);
+	   err=sparse_mat_write_file(&Ks[lvlm], name);
+	   FUNCTION_FAILURE_HANDLE( err, sparse_mat_write_file, main);
+
+	   {
+	     struct vector vx_old;
+	     FIDX i,d;
+
+	     err=vector_alloc(&vx_old, 2*vx_nr);
+	     FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
+	     for (i=0; i<vx_nr; i++)
+	       {
+		 for (d=0; d<2; d++)
+		   {
+		     vx_old.V[d*vx_nr+i]=msh2.vertex[i*msh2.vx_w+MCT2VXSTRT+d];
+		   }
+	       }
+	     
+	     /* node positions */
+	     sprintf(name,"visual/Lame_vertex_%s_%07"dFIDX"_nodes.txt",
+		     orderstring, vx_nr);
+	     err=vector_write_file(&vx_old, name);
+	     FUNCTION_FAILURE_HANDLE( err, vector_write_file, main);
+
+	   }
+	 }
 
       /* solve the equation system */
       if (set.solver==0) /* variant 1: direct solve */
@@ -608,6 +680,9 @@ int main(int argc, char *argv[])
                               16, "visual/Lame_sol" );
       FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
 			    main);
+      err=mesh_write_mesh_gnuplot_tx( &msh2, 17, "visual/Lame_mesh",2 );
+      FUNCTION_FAILURE_HANDLE( err, mesh_write_mesh_gnuplot_tx,
+			    main);
     }   
 
 
