@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -101,14 +101,10 @@ int main(int argc, char *argv[])
 
   dim=msh1.dim;
 
-  for (level=0; level<=levelmax; level ++)
-      { /* never do more than level 8 ! */
 
   /*printf("file read, vx_nr=%3d, eg_nr=%3d, fc_nr=%3d\n", msh1.vx_nr,
     msh1.eg_nr, msh1.fc_nr);*/
 
-  err=mesh_t1_to_t2( &msh1, &msh2 );
-  FUNCTION_FAILURE_HANDLE( err, mesh_t1_to_t2, main);
 
   for (lvl=0; lvl<0; lvl++) 
     {
@@ -117,13 +113,19 @@ int main(int argc, char *argv[])
     }
 
   /* reset hierarchy */
-  msh2.hi_nr=0;
-  msh2.eh_nr=0;
-  for (j=0; j<msh2.eg_nr; j++)
+  msh1.hi_nr=0;
+  msh1.lvl  =-1;
+  for (j=0; j<msh1.eg_nr; j++)
     {
-      msh2.edge[j*msh2.eg_w+MCT2EGLVL]=-1;
+      msh1.edge[j*msh1.eg_w+MCT1EGLVL]=-1;
     }
 
+  for (level=0; level<=levelmax; level ++)
+      { /* never do more than level 8 ! */
+
+  err=mesh_t1_to_t2( &msh1, &msh2 );
+  FUNCTION_FAILURE_HANDLE( err, mesh_t1_to_t2, main);
+
   err=navsto_matrix_init( &K, dim, level+1 );
   FUNCTION_FAILURE_HANDLE( err, navsto_matrix_init, main);
 
@@ -164,7 +166,12 @@ int main(int argc, char *argv[])
   TIMEGET;
   t2=ti;
 
-  vector_alloc( &y, x.len );
+  err=vector_alloc( &y, x.len );
+  FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
+
+  /* err=sparse_mat_write_file(&K.M, "visual/stokes_mass_mat.txt");
+     FUNCTION_FAILURE_HANDLE( err, sparse_mat_write_file, main); /* */
+
 
   resi=0.0;
   iter=0;
@@ -194,11 +201,11 @@ int main(int argc, char *argv[])
   K.mg->vccount=0;
   K.mg->stop_eps=K.innereps;
 
-  /* strcpy(solver,"PCR_w");
-     err=PCR( 100, 2, 0.0, 1e-5, 1, &x, &resi, &iter, stokes_matrix_tim_vec, 
-     stokes_projector_w_precon, &K, &rhs, NULL ); /* */
+  strcpy(solver,"PCR_w");
+  err=PCR( 100, 2, 0.0, 1e-5, 1, &x, &resi, &iter, stokes_matrix_tim_vec, 
+	   stokes_projector_w_precon, &K, &rhs, NULL ); /* */
 
-  strcpy(solver, "GMRES_w");
+  /* strcpy(solver, "GMRES_w");
   err=GMRES( m, 100, 2, 0.0, 1e-5, 1, &x, &resi, &iter,
 	     stokes_matrix_tim_vec, stokes_projector_w_precon,
 	     &K, &rhs, NULL); /* */
@@ -225,7 +232,7 @@ int main(int argc, char *argv[])
     }
   normy=sqrt(normy); /* */
   
-  printf("%8s: N=%6"dFIDX", it=%4d, err=%2d, |res_A|=%12.6e, |res_B|=%12.6e, ",
+  printf("%8s: N=%6"dFIDX", it=%4d, err=%2d, |res_A|=%9.3e, |res_B|=%9.3e, ",
 	 solver,  x.len, iter, err, resi, normy); /* */
   /* printf("%8s: N=%6d, it=%4d, err=%2d, |res_B|=%12.6e, ",
      solver, x.len, iter, err, normy); /* */
@@ -262,16 +269,16 @@ int main(int argc, char *argv[])
   }
 
   /* write the solution with grape */
-  soli.V=&sol.V[0];
-  soli.len=(dim+1)*msh2.vx_nr;
-  soli.n_max=(dim+1)*msh2.vx_nr;
-  err=mesh_write_solution_grape_t2( &msh2, &soli, dim+1, 20,
-				    "visual/stokes_grape_" );
-  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_grape_t2,
-			   test_stokes); /* */
+  /* soli.V=&sol.V[0];
+     soli.len=(dim+1)*msh2.vx_nr;
+     soli.n_max=(dim+1)*msh2.vx_nr;
+     err=mesh_write_solution_grape_t2( &msh2, &soli, dim+1, 20,
+     "visual/stokes_grape_" );
+     FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_grape_t2,
+     test_stokes); /* */
 
   /* reference solution */
-  TRY_MALLOC(buffer, strlen(argv[1])+7, char, main);
+  TRY_MALLOC(buffer, strlen(argv[1])+8, char, main);
   strcpy(buffer, argv[1]);
   strcat(buffer,"_stokes");
   err=vector_write_file(&x, buffer);
@@ -291,11 +298,12 @@ int main(int argc, char *argv[])
   vector_free(&x);
   vector_free(&y);
   vector_free(&sol);
-  mesh_free(&msh1);
   mesh_free(&msh2);
 
   } /* */
 
+  mesh_free(&msh1);
+
 
   return 1;
 }
